#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient import base


class Limit(base.Resource):
    """Represents a project limit.

    Attributes:
      * id: a UUID that identifies the project limit
      * service_id: a UUID that identifies the service for the limit
      * region_id: a UUID that identifies the region for the limit
      * project_id: a UUID that identifies the project for the limit
      * resource_name: the name of the resource to limit
      * resource_limit: the limit to apply to the project
      * description: a description for the project limit

    """

    pass


class LimitManager(base.CrudManager):
    """Manager class for project limits."""

    resource_class = Limit
    collection_key = 'limits'
    key = 'limit'

    def create(self, project, service, resource_name, resource_limit,
               description=None, region=None, **kwargs):
        """Create a project-specific limit.

        :param project: the project to create a limit for.
        :type project: str or :class:`keystoneclient.v3.projects.Project`
        :param service: the service that owns the resource to limit.
        :type service: str or :class:`keystoneclient.v3.services.Service`
        :param resource_name: the name of the resource to limit
        :type resource_name: str
        :param resource_limit: the quantity of the limit
        :type resource_limit: int
        :param description: a description of the limit
        :type description: str
        :param region: region the limit applies to
        :type region: str or :class:`keystoneclient.v3.regions.Region`

        :returns: a reference of the created limit
        :rtype: :class:`keystoneclient.v3.limits.Limit`

        """
        limit_data = base.filter_none(
            project_id=base.getid(project),
            service_id=base.getid(service),
            resource_name=resource_name,
            resource_limit=resource_limit,
            description=description,
            region_id=base.getid(region),
            **kwargs
        )
        body = {self.collection_key: [limit_data]}
        resp, body = self.client.post('/limits', body=body)
        limit = body[self.collection_key].pop()
        return self._prepare_return_value(resp,
                                          self.resource_class(
                                              self, limit))

    def update(self, limit, project=None, service=None, resource_name=None,
               resource_limit=None, description=None, **kwargs):
        """Update a project-specific limit.

        :param limit: a limit to update
        :param project: the project ID of the limit to update
        :type project: str or :class:`keystoneclient.v3.projects.Project`
        :param resource_limit: the limit of the limit's resource to update
        :type: resource_limit: int
        :param description: a description of the limit
        :type description: str

        :returns: a reference of the updated limit.
        :rtype: :class:`keystoneclient.v3.limits.Limit`

        """
        return super(LimitManager, self).update(
            limit_id=base.getid(limit),
            project_id=base.getid(project),
            service_id=base.getid(service),
            resource_name=resource_name,
            resource_limit=resource_limit,
            description=description,
            **kwargs
        )

    def get(self, limit):
        """Retrieve a project limit.

        :param limit:
            the project-specific limit to be retrieved.
        :type limit:
            str or :class:`keystoneclient.v3.limit.Limit`

        :returns: a project-specific limit
        :rtype: :class:`keystoneclient.v3.limit.Limit`

        """
        return super(LimitManager, self).get(limit_id=base.getid(limit))

    def list(self, service=None, region=None, resource_name=None, **kwargs):
        """List project-specific limits.

        Any parameter provided will be passed to the server as a filter

        :param service: service to filter limits by
        :type service: UUID or :class:`keystoneclient.v3.services.Service`
        :param region: region to filter limits by
        :type region: UUID or :class:`keystoneclient.v3.regions.Region`
        :param resource_name: the name of the resource to filter limits by
        :type resource_name: str

        :returns: a list of project-specific limits.
        :rtype: list of :class:`keystoneclient.v3.limits.Limit`

        """
        return super(LimitManager, self).list(
            service_id=base.getid(service),
            region_id=base.getid(region),
            resource_name=resource_name,
            **kwargs
        )

    def delete(self, limit):
        """Delete a project-specific limit.

        :param limit: the project-specific limit to be deleted.
        :type limit: str or :class:`keystoneclient.v3.limit.Limit`

        :returns: Response object with 204 status
        :rtype: :class:`requests.models.Response`

        """
        return super(LimitManager, self).delete(limit_id=base.getid(limit))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from keystoneauth1.exceptions import http

from keystoneclient.tests.functional import base
from keystoneclient.tests.functional.v3 import client_fixtures as fixtures


class EndpointsTestCase(base.V3ClientTestCase):

    def check_endpoint(self, endpoint, endpoint_ref=None):
        self.assertIsNotNone(endpoint.id)
        self.assertIn('self', endpoint.links)
        self.assertIn('/endpoints/' + endpoint.id, endpoint.links['self'])

        if endpoint_ref:
            self.assertEqual(endpoint_ref['service'], endpoint.service_id)
            self.assertEqual(endpoint_ref['url'], endpoint.url)
            self.assertEqual(endpoint_ref['interface'], endpoint.interface)
            self.assertEqual(endpoint_ref['enabled'], endpoint.enabled)

            # There is no guarantee below attributes are present in endpoint
            if hasattr(endpoint_ref, 'region'):
                self.assertEqual(endpoint_ref['region'], endpoint.region)

        else:
            # Only check remaining mandatory attributes
            self.assertIsNotNone(endpoint.service_id)
            self.assertIsNotNone(endpoint.url)
            self.assertIsNotNone(endpoint.interface)
            self.assertIsNotNone(endpoint.enabled)

    def test_create_endpoint(self):
        service = fixtures.Service(self.client)
        self.useFixture(service)

        endpoint_ref = {'service': service.id,
                        'url': 'http://' + uuid.uuid4().hex,
                        'enabled': True,
                        'interface': 'public'}
        endpoint = self.client.endpoints.create(**endpoint_ref)

        self.addCleanup(self.client.endpoints.delete, endpoint)
        self.check_endpoint(endpoint, endpoint_ref)

    def test_get_endpoint(self):
        service = fixtures.Service(self.client)
        self.useFixture(service)

        interfaces = ['public', 'admin', 'internal']
        for interface in interfaces:
            endpoint = fixtures.Endpoint(self.client, service.id, interface)
            self.useFixture(endpoint)
            endpoint_ret = self.client.endpoints.get(endpoint.id)
            # All endpoints are valid
            self.check_endpoint(endpoint_ret, endpoint.ref)

    def test_list_endpoints(self):
        service = fixtures.Service(self.client)
        self.useFixture(service)

        region = fixtures.Region(self.client)
        self.useFixture(region)

        endpoint_one = fixtures.Endpoint(self.client, service.id, 'public',
                                         region=region.id)
        self.useFixture(endpoint_one)

        endpoint_two = fixtures.Endpoint(self.client, service.id, 'admin',
                                         region=region.id)
        self.useFixture(endpoint_two)

        endpoint_three = fixtures.Endpoint(self.client, service.id, 'internal',
                                           region=region.id)
        self.useFixture(endpoint_three)

        endpoints = self.client.endpoints.list()

        # All endpoints are valid
        for endpoint in endpoints:
            self.check_endpoint(endpoint)

        self.assertIn(endpoint_one.entity, endpoints)
        self.assertIn(endpoint_two.entity, endpoints)
        self.assertIn(endpoint_three.entity, endpoints)

    def test_update_endpoint(self):
        service = fixtures.Service(self.client)
        self.useFixture(service)

        new_service = fixtures.Service(self.client)
        self.useFixture(new_service)

        new_region = fixtures.Region(self.client)
        self.useFixture(new_region)

        endpoint = fixtures.Endpoint(self.client, service.id, 'public')
        self.useFixture(endpoint)

        new_url = 'http://' + uuid.uuid4().hex
        new_interface = 'internal'
        new_enabled = False

        endpoint_ret = self.client.endpoints.update(endpoint.id,
                                                    service=new_service.id,
                                                    url=new_url,
                                                    interface=new_interface,
                                                    enabled=new_enabled,
                                                    region=new_region.id)

        endpoint.ref.update({'service': new_service.id, 'url': new_url,
                             'interface': new_interface,
                             'enabled': new_enabled,
                             'region': new_region.entity})
        self.check_endpoint(endpoint_ret, endpoint.ref)

    def test_delete_endpoint(self):
        service = fixtures.Service(self.client)
        self.useFixture(service)
        endpoint = self.client.endpoints.create(service=service.id,
                                                url='http://' +
                                                    uuid.uuid4().hex,
                                                enabled=True,
                                                interface='public')

        self.client.endpoints.delete(endpoint.id)
        self.assertRaises(http.NotFound,
                          self.client.endpoints.get,
                          endpoint.id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient import base


class ServiceProvider(base.Resource):
    """Object representing Service Provider container.

    Attributes:
        * id: user-defined unique string identifying Service Provider.
        * sp_url: the shibboleth endpoint of a Service Provider.
        * auth_url: the authentication url of Service Provider.

    """

    pass


class ServiceProviderManager(base.CrudManager):
    """Manager class for manipulating Service Providers."""

    resource_class = ServiceProvider
    collection_key = 'service_providers'
    key = 'service_provider'
    base_url = 'OS-FEDERATION'

    def _build_url_and_put(self, **kwargs):
        url = self.build_url(dict_args_in_out=kwargs)
        body = {self.key: kwargs}
        return self._update(url, body=body, response_key=self.key,
                            method='PUT')

    def create(self, id, **kwargs):
        """Create Service Provider object.

        Utilize Keystone URI:
        ``PUT /OS-FEDERATION/service_providers/{id}``

        :param id: unique id of the service provider.

        """
        return self._build_url_and_put(service_provider_id=id,
                                       **kwargs)

    def get(self, service_provider):
        """Fetch Service Provider object.

        Utilize Keystone URI:
        ``GET /OS-FEDERATION/service_providers/{id}``

        :param service_provider: an object with service_provider_id
                                 stored inside.

        """
        return super(ServiceProviderManager, self).get(
            service_provider_id=base.getid(service_provider))

    def list(self, **kwargs):
        """List all Service Providers.

        Utilize Keystone URI:
        ``GET /OS-FEDERATION/service_providers``

        """
        return super(ServiceProviderManager, self).list(**kwargs)

    def update(self, service_provider, **kwargs):
        """Update the existing Service Provider object on the server.

        Only properties provided to the function are being updated.

        Utilize Keystone URI:
        ``PATCH /OS-FEDERATION/service_providers/{id}``

        :param service_provider: an object with service_provider_id
                                 stored inside.

        """
        return super(ServiceProviderManager, self).update(
            service_provider_id=base.getid(service_provider), **kwargs)

    def delete(self, service_provider):
        """Delete Service Provider object.

        Utilize Keystone URI:
        ``DELETE /OS-FEDERATION/service_providers/{id}``

        :param service_provider: an object with service_provider_id
                                 stored inside.

        """
        return super(ServiceProviderManager, self).delete(
            service_provider_id=base.getid(service_provider))



# Copyright 2011 OpenStack Foundation
# Copyright 2011 Nebula, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneauth1 import plugin
import urllib.parse

from keystoneclient import base
from keystoneclient import exceptions


class Tenant(base.Resource):
    """Represents a Keystone tenant.

    Attributes:
        * id: a uuid that identifies the tenant
        * name: tenant name
        * description: tenant description
        * enabled: boolean to indicate if tenant is enabled

    """

    def __repr__(self):
        """Return string representation of tenant resource information."""
        return "<Tenant %s>" % self._info

    def delete(self):
        return self.manager.delete(self)

    def update(self, name=None, description=None, enabled=None):
        # Preserve the existing settings; keystone legacy resets these?
        new_name = name if name else self.name
        if description is not None:
            new_description = description
        else:
            new_description = self.description
        new_enabled = enabled if enabled is not None else self.enabled

        try:
            retval = self.manager.update(self.id, tenant_name=new_name,
                                         description=new_description,
                                         enabled=new_enabled)
            self = retval
        except Exception:
            retval = None
        return retval

    def add_user(self, user, role):
        return self.manager.role_manager.add_user_role(base.getid(user),
                                                       base.getid(role),
                                                       self.id)

    def remove_user(self, user, role):
        return self.manager.role_manager.remove_user_role(base.getid(user),
                                                          base.getid(role),
                                                          self.id)

    def list_users(self):
        return self.manager.list_users(self.id)


class TenantManager(base.ManagerWithFind):
    """Manager class for manipulating Keystone tenants."""

    resource_class = Tenant

    def __init__(self, client, role_manager, user_manager):
        super(TenantManager, self).__init__(client)
        self.role_manager = role_manager
        self.user_manager = user_manager

    def get(self, tenant_id):
        return self._get("/tenants/%s" % tenant_id, "tenant")

    def create(self, tenant_name, description=None, enabled=True, **kwargs):
        """Create a new tenant."""
        params = {"tenant": {"name": tenant_name,
                             "description": description,
                             "enabled": enabled}}

        # Allow Extras Passthru and ensure we don't clobber primary arguments.
        for k, v in kwargs.items():
            if k not in params['tenant']:
                params['tenant'][k] = v

        return self._post('/tenants', params, "tenant")

    def list(self, limit=None, marker=None):
        """Get a list of tenants.

        :param integer limit: maximum number to return. (optional)
        :param string marker: use when specifying a limit and making
                              multiple calls for querying. (optional)

        :rtype: list of :class:`Tenant`

        """
        params = {}
        if limit:
            params['limit'] = limit
        if marker:
            params['marker'] = marker

        query = ""
        if params:
            query = "?" + urllib.parse.urlencode(params)

        # NOTE(jamielennox): try doing a regular admin query first. If there is
        # no endpoint that can satisfy the request (eg an unscoped token) then
        # issue it against the auth_url.
        try:
            tenant_list = self._list('/tenants%s' % query, 'tenants')
        except exceptions.EndpointNotFound:
            endpoint_filter = {'interface': plugin.AUTH_INTERFACE}
            tenant_list = self._list('/tenants%s' % query, 'tenants',
                                     endpoint_filter=endpoint_filter)

        return tenant_list

    def update(self, tenant_id, tenant_name=None, description=None,
               enabled=None, **kwargs):
        """Update a tenant with a new name and description."""
        body = {"tenant": {'id': tenant_id}}
        if tenant_name is not None:
            body['tenant']['name'] = tenant_name
        if enabled is not None:
            body['tenant']['enabled'] = enabled
        if description is not None:
            body['tenant']['description'] = description

        # Allow Extras Passthru and ensure we don't clobber primary arguments.
        for k, v in kwargs.items():
            if k not in body['tenant']:
                body['tenant'][k] = v

        # Keystone's API uses a POST rather than a PUT here.
        return self._post("/tenants/%s" % tenant_id, body, "tenant")

    def delete(self, tenant):
        """Delete a tenant."""
        return self._delete("/tenants/%s" % (base.getid(tenant)))

    def list_users(self, tenant):
        """List users for a tenant."""
        return self.user_manager.list(base.getid(tenant))

    def add_user(self, tenant, user, role):
        """Add a user to a tenant with the given role."""
        return self.role_manager.add_user_role(base.getid(user),
                                               base.getid(role),
                                               base.getid(tenant))

    def remove_user(self, tenant, user, role):
        """Remove the specified role from the user on the tenant."""
        return self.role_manager.remove_user_role(base.getid(user),
                                                  base.getid(role),
                                                  base.getid(tenant))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Produce keystone compliant structures for testing.

The generators in this directory produce keystone compliant structures for
use in testing.
They should be considered part of the public API because they may be relied
upon to generate test tokens for other clients. However they should never be
imported into the main client (keystoneclient or other). Because of this there
may be dependencies from this module on libraries that are only available in
testing.

.. warning::

   The keystoneclient.fixture package is deprecated in favor of
   keystoneauth1.fixture and will not be supported.

"""

# flake8: noqa: F405

import warnings

from keystoneclient.fixture.discovery import *  # noqa
from keystoneclient.fixture import exception
from keystoneclient.fixture import v2
from keystoneclient.fixture import v3


warnings.warn(
    "The keystoneclient.fixture package is deprecated in favor of "
    "keystoneauth1.fixture and will not be supported.", DeprecationWarning)


FixtureValidationError = exception.FixtureValidationError
V2Token = v2.Token
V3Token = v3.Token
V3FederationToken = v3.V3FederationToken

__all__ = ('DiscoveryList',
           'FixtureValidationError',
           'V2Discovery',
           'V3Discovery',
           'V2Token',
           'V3Token',
           'V3FederationToken',
           )



# Copyright 2011 OpenStack Foundation
# Copyright 2011 Nebula, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient import base


class Service(base.Resource):
    """Represents a Keystone service."""

    def __repr__(self):
        """Return string representation of service resource information."""
        return "<Service %s>" % self._info


class ServiceManager(base.ManagerWithFind):
    """Manager class for manipulating Keystone services."""

    resource_class = Service

    def list(self):
        """List available services."""
        return self._list("/OS-KSADM/services", "OS-KSADM:services")

    def get(self, id):
        """Retrieve a service by id."""
        return self._get("/OS-KSADM/services/%s" % id, "OS-KSADM:service")

    def create(self, name, service_type, description=None):
        """Create a new service."""
        body = {"OS-KSADM:service": {'name': name,
                                     'type': service_type,
                                     'description': description}}
        return self._post("/OS-KSADM/services", body, "OS-KSADM:service")

    def delete(self, id):
        """Delete a service."""
        return self._delete("/OS-KSADM/services/%s" % id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_config import cfg

from keystoneclient import _discover
from keystoneclient.auth.identity.generic import base
from keystoneclient.auth.identity import v2
from keystoneclient.auth.identity import v3


def get_options():
    return [
        cfg.StrOpt('token', secret=True, help='Token to authenticate with'),
    ]


class Token(base.BaseGenericPlugin):
    """Generic token auth plugin.

    :param string token: Token for authentication.
    """

    def __init__(self, auth_url, token=None, **kwargs):
        super(Token, self).__init__(auth_url, **kwargs)
        self._token = token

    def create_plugin(self, session, version, url, raw_status=None):
        if _discover.version_match((2,), version):
            return v2.Token(url, self._token, **self._v2_params)

        elif _discover.version_match((3,), version):
            return v3.Token(url, self._token, **self._v3_params)

    @classmethod
    def get_options(cls):
        options = super(Token, cls).get_options()
        options.extend(get_options())
        return options



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

from keystoneclient.tests.unit.v3 import utils
from keystoneclient.v3 import domains


class DomainTests(utils.ClientTestCase, utils.CrudTests):
    def setUp(self):
        super(DomainTests, self).setUp()
        self.key = 'domain'
        self.collection_key = 'domains'
        self.model = domains.Domain
        self.manager = self.client.domains

    def new_ref(self, **kwargs):
        kwargs = super(DomainTests, self).new_ref(**kwargs)
        kwargs.setdefault('enabled', True)
        kwargs.setdefault('name', uuid.uuid4().hex)
        return kwargs

    def test_filter_for_default_domain_by_id(self):
        ref = self.new_ref(id='default')
        super(DomainTests, self).test_list_by_id(
            ref=ref,
            id=ref['id'])

    def test_list_filter_name(self):
        super(DomainTests, self).test_list(name='adomain123')

    def test_list_filter_enabled(self):
        super(DomainTests, self).test_list(enabled=True)

    def test_list_filter_disabled(self):
        # False is converted to '0' ref bug #1267530
        expected_query = {'enabled': '0'}
        super(DomainTests, self).test_list(expected_query=expected_query,
                                           enabled=False)

    def test_update_enabled_defaults_to_none(self):
        super(DomainTests, self).test_update(
            req_ref={'name': uuid.uuid4().hex})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import uuid

from keystoneauth1 import fixture


def unscoped_token(**kwargs):
    return fixture.V3Token(**kwargs)


def domain_scoped_token(**kwargs):
    kwargs.setdefault('audit_chain_id', uuid.uuid4().hex)
    f = fixture.V3Token(**kwargs)
    if not f.domain_id:
        f.set_domain_scope()

    f.add_role(name='admin')
    f.add_role(name='member')
    region = 'RegionOne'

    s = f.add_service('volume')
    s.add_standard_endpoints(public='http://public.com:8776/v1/None',
                             internal='http://internal.com:8776/v1/None',
                             admin='http://admin.com:8776/v1/None',
                             region=region)

    s = f.add_service('image')
    s.add_standard_endpoints(public='http://public.com:9292/v1',
                             internal='http://internal:9292/v1',
                             admin='http://admin:9292/v1',
                             region=region)

    s = f.add_service('compute')
    s.add_standard_endpoints(public='http://public.com:8774/v1.1/None',
                             internal='http://internal:8774/v1.1/None',
                             admin='http://admin:8774/v1.1/None',
                             region=region)

    s = f.add_service('ec2')
    s.add_standard_endpoints(public='http://public.com:8773/services/Cloud',
                             internal='http://internal:8773/services/Cloud',
                             admin='http://admin:8773/services/Admin',
                             region=region)

    s = f.add_service('identity')
    s.add_standard_endpoints(public='http://public.com:5000/v3',
                             internal='http://internal:5000/v3',
                             admin='http://admin:35357/v3',
                             region=region)

    return f


def project_scoped_token(**kwargs):
    kwargs.setdefault('audit_chain_id', uuid.uuid4().hex)
    f = fixture.V3Token(**kwargs)

    if not f.project_id:
        f.set_project_scope()

    f.add_role(name='admin')
    f.add_role(name='member')

    region = 'RegionOne'
    tenant = '225da22d3ce34b15877ea70b2a575f58'

    s = f.add_service('volume')
    s.add_standard_endpoints(public='http://public.com:8776/v1/%s' % tenant,
                             internal='http://internal:8776/v1/%s' % tenant,
                             admin='http://admin:8776/v1/%s' % tenant,
                             region=region)

    s = f.add_service('image')
    s.add_standard_endpoints(public='http://public.com:9292/v1',
                             internal='http://internal:9292/v1',
                             admin='http://admin:9292/v1',
                             region=region)

    s = f.add_service('compute')
    s.add_standard_endpoints(public='http://public.com:8774/v2/%s' % tenant,
                             internal='http://internal:8774/v2/%s' % tenant,
                             admin='http://admin:8774/v2/%s' % tenant,
                             region=region)

    s = f.add_service('ec2')
    s.add_standard_endpoints(public='http://public.com:8773/services/Cloud',
                             internal='http://internal:8773/services/Cloud',
                             admin='http://admin:8773/services/Admin',
                             region=region)

    s = f.add_service('identity')
    s.add_standard_endpoints(public='http://public.com:5000/v3',
                             internal='http://internal:5000/v3',
                             admin='http://admin:35357/v3',
                             region=region)

    return f


AUTH_SUBJECT_TOKEN = uuid.uuid4().hex

AUTH_RESPONSE_HEADERS = {
    'X-Subject-Token': AUTH_SUBJECT_TOKEN,
}


def auth_response_body():
    f = fixture.V3Token(audit_chain_id=uuid.uuid4().hex)
    f.set_project_scope()

    f.add_role(name='admin')
    f.add_role(name='member')

    s = f.add_service('compute', name='nova')
    s.add_standard_endpoints(
        public='https://compute.north.host/novapi/public',
        internal='https://compute.north.host/novapi/internal',
        admin='https://compute.north.host/novapi/admin',
        region='North')

    s = f.add_service('object-store', name='swift')
    s.add_standard_endpoints(
        public='http://swift.north.host/swiftapi/public',
        internal='http://swift.north.host/swiftapi/internal',
        admin='http://swift.north.host/swiftapi/admin',
        region='South')

    s = f.add_service('image', name='glance')
    s.add_standard_endpoints(
        public='http://glance.north.host/glanceapi/public',
        internal='http://glance.north.host/glanceapi/internal',
        admin='http://glance.north.host/glanceapi/admin',
        region='North')

    s.add_standard_endpoints(
        public='http://glance.south.host/glanceapi/public',
        internal='http://glance.south.host/glanceapi/internal',
        admin='http://glance.south.host/glanceapi/admin',
        region='South')

    return f


def trust_token():
    f = fixture.V3Token(audit_chain_id=uuid.uuid4().hex)
    f.set_trust_scope()
    return f



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneauth1.fixture import discovery


__all__ = ('DiscoveryList',
           'V2Discovery',
           'V3Discovery',
           )


V2Discovery = discovery.V2Discovery
"""A Version element for a V2 identity service endpoint.

An alias of :py:exc:`keystoneauth1.fixture.discovery.V2Discovery`
"""

V3Discovery = discovery.V3Discovery
"""A Version element for a V3 identity service endpoint.

An alias of :py:exc:`keystoneauth1.fixture.discovery.V3Discovery`
"""

DiscoveryList = discovery.DiscoveryList
"""A List of version elements.

An alias of :py:exc:`keystoneauth1.fixture.discovery.DiscoveryList`
"""



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import errno
import os
import subprocess
from unittest import mock

import testresources
from testtools import matchers

from keystoneclient.common import cms
from keystoneclient import exceptions
from keystoneclient.tests.unit import client_fixtures
from keystoneclient.tests.unit import utils


class CMSTest(utils.TestCase, testresources.ResourcedTestCase):
    """Unit tests for the keystoneclient.common.cms module."""

    resources = [('examples', client_fixtures.EXAMPLES_RESOURCE)]

    def __init__(self, *args, **kwargs):
        super(CMSTest, self).__init__(*args, **kwargs)
        process = subprocess.Popen(['openssl', 'version'],
                                   stdout=subprocess.PIPE)
        out, err = process.communicate()
        # Example output: 'OpenSSL 0.9.8za 5 Jun 2014'
        openssl_version = out.split()[1]

        if err or openssl_version.startswith(b'0'):
            raise Exception('Your version of OpenSSL is not supported. '
                            'You will need to update it to 1.0 or later.')

    def _raise_OSError(*args):
        e = OSError()
        e.errno = errno.EPIPE
        raise e

    def test_cms_verify(self):
        self.assertRaises(exceptions.CertificateConfigError,
                          cms.cms_verify,
                          'data',
                          'no_exist_cert_file',
                          'no_exist_ca_file')

    def test_token_tocms_to_token(self):
        with open(os.path.join(client_fixtures.CMSDIR,
                               'auth_token_scoped.pem')) as f:
            AUTH_TOKEN_SCOPED_CMS = f.read()

        self.assertEqual(cms.token_to_cms(self.examples.SIGNED_TOKEN_SCOPED),
                         AUTH_TOKEN_SCOPED_CMS)

        tok = cms.cms_to_token(cms.token_to_cms(
            self.examples.SIGNED_TOKEN_SCOPED))
        self.assertEqual(tok, self.examples.SIGNED_TOKEN_SCOPED)

    def test_asn1_token(self):
        self.assertTrue(cms.is_asn1_token(self.examples.SIGNED_TOKEN_SCOPED))
        self.assertFalse(cms.is_asn1_token('FOOBAR'))

    def test_cms_sign_token_no_files(self):
        self.assertRaises(subprocess.CalledProcessError,
                          cms.cms_sign_token,
                          self.examples.TOKEN_SCOPED_DATA,
                          '/no/such/file', '/no/such/key')

    def test_cms_sign_token_no_files_pkiz(self):
        self.assertRaises(subprocess.CalledProcessError,
                          cms.pkiz_sign,
                          self.examples.TOKEN_SCOPED_DATA,
                          '/no/such/file', '/no/such/key')

    def test_cms_sign_token_success(self):
        self.assertTrue(
            cms.pkiz_sign(self.examples.TOKEN_SCOPED_DATA,
                          self.examples.SIGNING_CERT_FILE,
                          self.examples.SIGNING_KEY_FILE))

    def test_cms_verify_token_no_files(self):
        self.assertRaises(exceptions.CertificateConfigError,
                          cms.cms_verify,
                          self.examples.SIGNED_TOKEN_SCOPED,
                          '/no/such/file', '/no/such/key')

    def test_cms_verify_token_no_oserror(self):
        with mock.patch('subprocess.Popen.communicate',
                        new=self._raise_OSError):
            try:
                cms.cms_verify("x", '/no/such/file', '/no/such/key')
            except exceptions.CertificateConfigError as e:
                self.assertIn('/no/such/file', e.output)
                self.assertIn('Hit OSError ', e.output)
            else:
                self.fail('Expected exceptions.CertificateConfigError')

    def test_cms_verify_token_scoped(self):
        cms_content = cms.token_to_cms(self.examples.SIGNED_TOKEN_SCOPED)
        self.assertTrue(cms.cms_verify(cms_content,
                                       self.examples.SIGNING_CERT_FILE,
                                       self.examples.SIGNING_CA_FILE))

    def test_cms_verify_token_scoped_expired(self):
        cms_content = cms.token_to_cms(
            self.examples.SIGNED_TOKEN_SCOPED_EXPIRED)
        self.assertTrue(cms.cms_verify(cms_content,
                                       self.examples.SIGNING_CERT_FILE,
                                       self.examples.SIGNING_CA_FILE))

    def test_cms_verify_token_unscoped(self):
        cms_content = cms.token_to_cms(self.examples.SIGNED_TOKEN_UNSCOPED)
        self.assertTrue(cms.cms_verify(cms_content,
                                       self.examples.SIGNING_CERT_FILE,
                                       self.examples.SIGNING_CA_FILE))

    def test_cms_verify_token_v3_scoped(self):
        cms_content = cms.token_to_cms(self.examples.SIGNED_v3_TOKEN_SCOPED)
        self.assertTrue(cms.cms_verify(cms_content,
                                       self.examples.SIGNING_CERT_FILE,
                                       self.examples.SIGNING_CA_FILE))

    def test_cms_hash_token_no_token_id(self):
        token_id = None
        self.assertThat(cms.cms_hash_token(token_id), matchers.Is(None))

    def test_cms_hash_token_not_pki(self):
        """If the token_id is not a PKI token then it returns the token_id."""
        token = 'something'
        self.assertFalse(cms.is_asn1_token(token))
        self.assertThat(cms.cms_hash_token(token), matchers.Is(token))

    def test_cms_hash_token_default_md5(self):
        """The default hash method is md5."""
        token = self.examples.SIGNED_TOKEN_SCOPED
        token_id_default = cms.cms_hash_token(token)
        token_id_md5 = cms.cms_hash_token(token, mode='md5')
        self.assertThat(token_id_default, matchers.Equals(token_id_md5))
        # md5 hash is 32 chars.
        self.assertThat(token_id_default, matchers.HasLength(32))

    def test_cms_hash_token_sha256(self):
        """Can also hash with sha256."""
        token = self.examples.SIGNED_TOKEN_SCOPED
        token_id = cms.cms_hash_token(token, mode='sha256')
        # sha256 hash is 64 chars.
        self.assertThat(token_id, matchers.HasLength(64))

    @mock.patch('keystoneclient.common.cms._check_files_accessible')
    def test_process_communicate_handle_oserror_epipe(self, files_acc_mock):
        process_mock = mock.Mock()
        process_mock.communicate = self._raise_OSError
        process_mock.stderr = mock.Mock()
        process_mock.stderr.read = mock.Mock(return_value='proc stderr')
        files_acc_mock.return_value = 1, ('file_path', 'fileerror')
        output, err, retcode = cms._process_communicate_handle_oserror(
            process_mock, '', [])

        self.assertEqual((output, retcode), ('', 1))
        self.assertIn('file_path', err)
        self.assertIn('fileerror', err)
        self.assertIn('proc stderr', err)

    @mock.patch('keystoneclient.common.cms._check_files_accessible')
    def test_process_communicate_handle_oserror_epipe_files_ok(
            self, files_acc_mock):
        process_mock = mock.Mock()
        process_mock.communicate = self._raise_OSError
        process_mock.stderr = mock.Mock()
        process_mock.stderr.read = mock.Mock(return_value='proc stderr')
        files_acc_mock.return_value = -1, None
        output, err, retcode = cms._process_communicate_handle_oserror(
            process_mock, '', [])

        self.assertEqual((output, retcode), ('', -1))
        self.assertIn('proc stderr', err)

    def test_process_communicate_handle_oserror_no_exception(self):
        process_mock = mock.Mock()
        process_mock.communicate.return_value = 'out', 'err'
        process_mock.poll.return_value = 0

        output, err, retcode = cms._process_communicate_handle_oserror(
            process_mock, '', [])

        self.assertEqual(output, 'out')
        self.assertEqual(err, 'err')
        self.assertEqual(retcode, 0)


def load_tests(loader, tests, pattern):
    return testresources.OptimisingTestSuite(tests)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from keystoneauth1.exceptions import http

from keystoneclient.tests.functional import base
from keystoneclient.tests.functional.v3 import client_fixtures as fixtures


class PoliciesTestCase(base.V3ClientTestCase):

    def check_policy(self, policy, policy_ref=None):
        self.assertIsNotNone(policy.id)
        self.assertIn('self', policy.links)
        self.assertIn('/policies/' + policy.id, policy.links['self'])

        if policy_ref:
            self.assertEqual(policy_ref['blob'], policy.blob)
            self.assertEqual(policy_ref['type'], policy.type)

        else:
            # Only check remaining mandatory attributes
            self.assertIsNotNone(policy.blob)
            self.assertIsNotNone(policy.type)

    def test_create_policy(self):
        policy_ref = {'blob': uuid.uuid4().hex,
                      'type': uuid.uuid4().hex}

        policy = self.client.policies.create(**policy_ref)
        self.addCleanup(self.client.policies.delete, policy)
        self.check_policy(policy, policy_ref)

    def test_get_policy(self):
        policy = fixtures.Policy(self.client)
        self.useFixture(policy)

        policy_ret = self.client.policies.get(policy.id)
        self.check_policy(policy_ret, policy.ref)

    def test_list_policies(self):
        policy_one = fixtures.Policy(self.client)
        self.useFixture(policy_one)

        policy_two = fixtures.Policy(self.client)
        self.useFixture(policy_two)

        policies = self.client.policies.list()

        # All policies are valid
        for policy in policies:
            self.check_policy(policy)

        self.assertIn(policy_one.entity, policies)
        self.assertIn(policy_two.entity, policies)

    def test_update_policy(self):
        policy = fixtures.Policy(self.client)
        self.useFixture(policy)

        new_blob = uuid.uuid4().hex
        new_type = uuid.uuid4().hex

        policy_ret = self.client.policies.update(policy.id,
                                                 blob=new_blob,
                                                 type=new_type)

        policy.ref.update({'blob': new_blob, 'type': new_type})
        self.check_policy(policy_ret, policy.ref)

    def test_delete_policy(self):
        policy = self.client.policies.create(blob=uuid.uuid4().hex,
                                             type=uuid.uuid4().hex)

        self.client.policies.delete(policy.id)
        self.assertRaises(http.NotFound,
                          self.client.policies.get,
                          policy.id)



# Copyright 2013 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import io
import logging

from testtools import matchers

from keystoneclient import exceptions
from keystoneclient import httpclient
from keystoneclient import session
from keystoneclient.tests.unit import utils


RESPONSE_BODY = '{"hi": "there"}'


def get_client():
    cl = httpclient.HTTPClient(username="username", password="password",
                               project_id="tenant", auth_url="auth_test")
    return cl


def get_authed_client():
    cl = get_client()
    cl.management_url = "http://127.0.0.1:5000"
    cl.auth_token = "token"
    return cl


class ClientTest(utils.TestCase):

    TEST_URL = 'http://127.0.0.1:5000/hi'

    def test_unauthorized_client_requests(self):
        # Creating a HTTPClient not using session is deprecated.
        with self.deprecations.expect_deprecations_here():
            cl = get_client()
        self.assertRaises(exceptions.AuthorizationFailure, cl.get, '/hi')
        self.assertRaises(exceptions.AuthorizationFailure, cl.post, '/hi')
        self.assertRaises(exceptions.AuthorizationFailure, cl.put, '/hi')
        self.assertRaises(exceptions.AuthorizationFailure, cl.delete, '/hi')

    def test_get(self):
        # Creating a HTTPClient not using session is deprecated.
        with self.deprecations.expect_deprecations_here():
            cl = get_authed_client()

        self.stub_url('GET', text=RESPONSE_BODY)

        with self.deprecations.expect_deprecations_here():
            resp, body = cl.get("/hi")
        self.assertEqual(self.requests_mock.last_request.method, 'GET')
        self.assertEqual(self.requests_mock.last_request.url, self.TEST_URL)

        self.assertRequestHeaderEqual('X-Auth-Token', 'token')
        self.assertRequestHeaderEqual('User-Agent', httpclient.USER_AGENT)

        # Automatic JSON parsing
        self.assertEqual(body, {"hi": "there"})

    def test_get_error_with_plaintext_resp(self):
        # Creating a HTTPClient not using session is deprecated.
        with self.deprecations.expect_deprecations_here():
            cl = get_authed_client()
        self.stub_url('GET', status_code=400,
                      text='Some evil plaintext string')

        self.assertRaises(exceptions.BadRequest, cl.get, '/hi')

    def test_get_error_with_json_resp(self):
        # Creating a HTTPClient not using session is deprecated.
        with self.deprecations.expect_deprecations_here():
            cl = get_authed_client()
        err_response = {
            "error": {
                "code": 400,
                "title": "Error title",
                "message": "Error message string"
            }
        }
        self.stub_url('GET', status_code=400, json=err_response)
        exc_raised = False
        try:
            with self.deprecations.expect_deprecations_here():
                cl.get('/hi')
        except exceptions.BadRequest as exc:
            exc_raised = True
            self.assertEqual(exc.message, "Error message string (HTTP 400)")
        self.assertTrue(exc_raised, 'Exception not raised.')

    def test_post(self):
        # Creating a HTTPClient not using session is deprecated.
        with self.deprecations.expect_deprecations_here():
            cl = get_authed_client()

        self.stub_url('POST')
        with self.deprecations.expect_deprecations_here():
            cl.post("/hi", body=[1, 2, 3])

        self.assertEqual(self.requests_mock.last_request.method, 'POST')
        self.assertEqual(self.requests_mock.last_request.body, '[1, 2, 3]')

        self.assertRequestHeaderEqual('X-Auth-Token', 'token')
        self.assertRequestHeaderEqual('Content-Type', 'application/json')
        self.assertRequestHeaderEqual('User-Agent', httpclient.USER_AGENT)

    def test_forwarded_for(self):
        ORIGINAL_IP = "10.100.100.1"
        # Creating a HTTPClient not using session is deprecated.
        with self.deprecations.expect_deprecations_here():
            cl = httpclient.HTTPClient(username="username",
                                       password="password",
                                       project_id="tenant",
                                       auth_url="auth_test",
                                       original_ip=ORIGINAL_IP)

        self.stub_url('GET')

        with self.deprecations.expect_deprecations_here():
            cl.request(self.TEST_URL, 'GET')
        forwarded = "for=%s;by=%s" % (ORIGINAL_IP, httpclient.USER_AGENT)
        self.assertRequestHeaderEqual('Forwarded', forwarded)

    def test_client_deprecated(self):
        # Can resolve symbols from the keystoneclient.client module.
        # keystoneclient.client was deprecated and renamed to
        # keystoneclient.httpclient. This tests that keystoneclient.client
        # can still be used.

        from keystoneclient import client

        # These statements will raise an AttributeError if the symbol isn't
        # defined in the module.

        client.HTTPClient


class BasicRequestTests(utils.TestCase):

    url = 'http://keystone.test.com/'

    def setUp(self):
        super(BasicRequestTests, self).setUp()
        self.logger_message = io.StringIO()
        handler = logging.StreamHandler(self.logger_message)
        handler.setLevel(logging.DEBUG)

        self.logger = logging.getLogger(session.__name__)
        level = self.logger.getEffectiveLevel()
        self.logger.setLevel(logging.DEBUG)
        self.logger.addHandler(handler)

        self.addCleanup(self.logger.removeHandler, handler)
        self.addCleanup(self.logger.setLevel, level)

    def request(self, method='GET', response='Test Response', status_code=200,
                url=None, headers={}, **kwargs):
        if not url:
            url = self.url

        self.requests_mock.register_uri(method, url, text=response,
                                        status_code=status_code,
                                        headers=headers)

        with self.deprecations.expect_deprecations_here():
            return httpclient.request(url, method, headers=headers, **kwargs)

    def test_basic_params(self):
        method = 'GET'
        response = 'Test Response'
        status = 200

        self.request(method=method, status_code=status, response=response,
                     headers={'Content-Type': 'application/json'})

        self.assertEqual(self.requests_mock.last_request.method, method)

        logger_message = self.logger_message.getvalue()

        self.assertThat(logger_message, matchers.Contains('curl'))
        self.assertThat(logger_message, matchers.Contains('-X %s' %
                                                          method))
        self.assertThat(logger_message, matchers.Contains(self.url))

        self.assertThat(logger_message, matchers.Contains(str(status)))
        self.assertThat(logger_message, matchers.Contains(response))

    def test_headers(self):
        headers = {'key': 'val', 'test': 'other'}

        self.request(headers=headers)

        for k, v in headers.items():
            self.assertRequestHeaderEqual(k, v)

        for header in headers.items():
            self.assertThat(self.logger_message.getvalue(),
                            matchers.Contains('-H "%s: %s"' % header))

    def test_body(self):
        data = "BODY DATA"
        self.request(response=data,
                     headers={'Content-Type': 'application/json'})
        logger_message = self.logger_message.getvalue()
        self.assertThat(logger_message, matchers.Contains('BODY:'))
        self.assertThat(logger_message, matchers.Contains(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import functools
from unittest import mock
import uuid

from keystoneauth1 import fixture
from oslo_config import cfg

from keystoneclient import access
from keystoneclient.auth import base
from keystoneclient import exceptions
from keystoneclient import session
from keystoneclient.tests.unit import utils


class MockPlugin(base.BaseAuthPlugin):

    INT_DESC = 'test int'
    FLOAT_DESC = 'test float'
    BOOL_DESC = 'test bool'
    STR_DESC = 'test str'
    STR_DEFAULT = uuid.uuid4().hex

    def __init__(self, **kwargs):
        self._data = kwargs

    def __getitem__(self, key):
        """Get the data of the key."""
        return self._data[key]

    def get_token(self, *args, **kwargs):
        return 'aToken'

    def get_endpoint(self, *args, **kwargs):
        return 'http://test'

    @classmethod
    def get_options(cls):
        return [
            cfg.IntOpt('a-int', default='3', help=cls.INT_DESC),
            cfg.BoolOpt('a-bool', help=cls.BOOL_DESC),
            cfg.FloatOpt('a-float', help=cls.FLOAT_DESC),
            cfg.StrOpt('a-str', help=cls.STR_DESC, default=cls.STR_DEFAULT),
        ]


class MockManager(object):

    def __init__(self, driver):
        self.driver = driver


def mock_plugin(f):
    @functools.wraps(f)
    def inner(*args, **kwargs):
        with mock.patch.object(base, 'get_plugin_class') as m:
            m.return_value = MockPlugin
            args = list(args) + [m]
            return f(*args, **kwargs)

    return inner


class TestCase(utils.TestCase):

    GROUP = 'auth'
    V2PASS = 'v2password'
    V3TOKEN = 'v3token'

    a_int = 88
    a_float = 88.8
    a_bool = False

    TEST_VALS = {'a_int': a_int,
                 'a_float': a_float,
                 'a_bool': a_bool}

    def assertTestVals(self, plugin, vals=TEST_VALS):
        for k, v in vals.items():
            self.assertEqual(v, plugin[k])


class GenericPluginTestCase(utils.TestCase):

    TEST_URL = 'http://keystone.host:5000/'

    # OVERRIDE THESE IN SUB CLASSES
    PLUGIN_CLASS = None
    V2_PLUGIN_CLASS = None
    V3_PLUGIN_CLASS = None

    def setUp(self):
        super(GenericPluginTestCase, self).setUp()

        self.token_v2 = fixture.V2Token()
        self.token_v3 = fixture.V3Token()
        self.token_v3_id = uuid.uuid4().hex

        self.deprecations.expect_deprecations()
        self.session = session.Session()

        self.stub_url('POST', ['v2.0', 'tokens'], json=self.token_v2)
        self.stub_url('POST', ['v3', 'auth', 'tokens'],
                      headers={'X-Subject-Token': self.token_v3_id},
                      json=self.token_v3)

    def new_plugin(self, **kwargs):
        kwargs.setdefault('auth_url', self.TEST_URL)
        return self.PLUGIN_CLASS(**kwargs)

    def stub_discovery(self, base_url=None, **kwargs):
        kwargs.setdefault('href', self.TEST_URL)
        disc = fixture.DiscoveryList(**kwargs)
        self.stub_url('GET', json=disc, base_url=base_url, status_code=300)
        return disc

    def assertCreateV3(self, **kwargs):
        auth = self.new_plugin(**kwargs)
        auth_ref = auth.get_auth_ref(self.session)
        self.assertIsInstance(auth_ref, access.AccessInfoV3)
        self.assertEqual(self.TEST_URL + 'v3/auth/tokens',
                         self.requests_mock.last_request.url)
        self.assertIsInstance(auth._plugin, self.V3_PLUGIN_CLASS)
        return auth

    def assertCreateV2(self, **kwargs):
        auth = self.new_plugin(**kwargs)
        auth_ref = auth.get_auth_ref(self.session)
        self.assertIsInstance(auth_ref, access.AccessInfoV2)
        self.assertEqual(self.TEST_URL + 'v2.0/tokens',
                         self.requests_mock.last_request.url)
        self.assertIsInstance(auth._plugin, self.V2_PLUGIN_CLASS)
        return auth

    def assertDiscoveryFailure(self, **kwargs):
        plugin = self.new_plugin(**kwargs)
        self.assertRaises(exceptions.DiscoveryFailure,
                          plugin.get_auth_ref,
                          self.session)

    def test_create_v3_if_domain_params(self):
        self.stub_discovery()

        self.assertCreateV3(domain_id=uuid.uuid4().hex)
        self.assertCreateV3(domain_name=uuid.uuid4().hex)
        self.assertCreateV3(project_name=uuid.uuid4().hex,
                            project_domain_name=uuid.uuid4().hex)
        self.assertCreateV3(project_name=uuid.uuid4().hex,
                            project_domain_id=uuid.uuid4().hex)

    def test_create_v2_if_no_domain_params(self):
        self.stub_discovery()
        self.assertCreateV2()
        self.assertCreateV2(project_id=uuid.uuid4().hex)
        self.assertCreateV2(project_name=uuid.uuid4().hex)
        self.assertCreateV2(tenant_id=uuid.uuid4().hex)
        self.assertCreateV2(tenant_name=uuid.uuid4().hex)

    def test_v3_params_v2_url(self):
        self.stub_discovery(v3=False)
        self.assertDiscoveryFailure(domain_name=uuid.uuid4().hex)

    def test_v2_params_v3_url(self):
        self.stub_discovery(v2=False)
        self.assertCreateV3()

    def test_no_urls(self):
        self.stub_discovery(v2=False, v3=False)
        self.assertDiscoveryFailure()

    def test_path_based_url_v2(self):
        self.stub_url('GET', ['v2.0'], status_code=403)
        self.assertCreateV2(auth_url=self.TEST_URL + 'v2.0')

    def test_path_based_url_v3(self):
        self.stub_url('GET', ['v3'], status_code=403)
        self.assertCreateV3(auth_url=self.TEST_URL + 'v3')

    def test_disc_error_for_failure(self):
        self.stub_url('GET', [], status_code=403)
        self.assertDiscoveryFailure()

    def test_v3_plugin_from_failure(self):
        url = self.TEST_URL + 'v3'
        self.stub_url('GET', [], base_url=url, status_code=403)
        self.assertCreateV3(auth_url=url)

    def test_unknown_discovery_version(self):
        # make a v4 entry that's mostly the same as a v3
        self.stub_discovery(v2=False, v3_id='v4.0')
        self.assertDiscoveryFailure()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

from keystoneclient.tests.unit.v3 import utils
from keystoneclient.v3 import endpoint_groups


class EndpointGroupTests(utils.ClientTestCase, utils.CrudTests):

    def setUp(self):
        super(EndpointGroupTests, self).setUp()
        self.key = 'endpoint_group'
        self.collection_key = 'endpoint_groups'
        self.model = endpoint_groups.EndpointGroup
        self.manager = self.client.endpoint_groups
        self.path_prefix = 'OS-EP-FILTER'

    def new_ref(self, **kwargs):
        kwargs.setdefault('id', uuid.uuid4().hex)
        kwargs.setdefault('name', uuid.uuid4().hex)
        kwargs.setdefault('filters', '{"interface": "public"}')
        kwargs.setdefault('description', uuid.uuid4().hex)
        return kwargs



# Copyright 2012 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from keystoneclient import exceptions
from keystoneclient.tests.unit.v3 import utils
from keystoneclient.v3 import roles
from testtools import matchers


class RoleTests(utils.ClientTestCase, utils.CrudTests):
    def setUp(self):
        super(RoleTests, self).setUp()
        self.key = 'role'
        self.collection_key = 'roles'
        self.model = roles.Role
        self.manager = self.client.roles

    def new_ref(self, **kwargs):
        kwargs = super(RoleTests, self).new_ref(**kwargs)
        kwargs.setdefault('name', uuid.uuid4().hex)
        return kwargs

    def _new_domain_ref(self, **kwargs):
        kwargs.setdefault('enabled', True)
        kwargs.setdefault('name', uuid.uuid4().hex)
        return kwargs

    def test_create_with_domain_id(self):
        ref = self.new_ref()
        ref['domain_id'] = uuid.uuid4().hex
        self.test_create(ref=ref)

    def test_create_with_domain(self):
        ref = self.new_ref()
        domain_ref = self._new_domain_ref()
        domain_ref['id'] = uuid.uuid4().hex
        ref['domain_id'] = domain_ref['id']

        self.stub_entity('POST', entity=ref, status_code=201)
        returned = self.manager.create(name=ref['name'],
                                       domain=domain_ref)
        self.assertIsInstance(returned, self.model)
        for attr in ref:
            self.assertEqual(
                getattr(returned, attr),
                ref[attr],
                'Expected different %s' % attr)

    def test_domain_role_grant(self):
        user_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('PUT',
                      ['domains', domain_id, 'users', user_id,
                       self.collection_key, ref['id']],
                      status_code=201)

        self.manager.grant(role=ref['id'], domain=domain_id, user=user_id)

    def test_domain_role_grant_inherited(self):
        user_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('PUT',
                      ['OS-INHERIT', 'domains', domain_id, 'users', user_id,
                       self.collection_key, ref['id'],
                       'inherited_to_projects'],
                      status_code=201)

        self.manager.grant(role=ref['id'], domain=domain_id, user=user_id,
                           os_inherit_extension_inherited=True)

    def test_project_role_grant_inherited(self):
        user_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('PUT',
                      ['OS-INHERIT', 'projects', project_id, 'users', user_id,
                       self.collection_key, ref['id'],
                       'inherited_to_projects'],
                      status_code=204)

        self.manager.grant(role=ref['id'], project=project_id, user=user_id,
                           os_inherit_extension_inherited=True)

    def test_domain_group_role_grant(self):
        group_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('PUT',
                      ['domains', domain_id, 'groups', group_id,
                       self.collection_key, ref['id']],
                      status_code=201)

        self.manager.grant(role=ref['id'], domain=domain_id, group=group_id)

    def test_domain_group_role_grant_inherited(self):
        group_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('PUT',
                      ['OS-INHERIT', 'domains', domain_id, 'groups', group_id,
                       self.collection_key, ref['id'],
                       'inherited_to_projects'],
                      status_code=201)

        self.manager.grant(role=ref['id'], domain=domain_id, group=group_id,
                           os_inherit_extension_inherited=True)

    def test_project_group_role_grant_inherited(self):
        group_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('PUT',
                      ['OS-INHERIT', 'projects', project_id, 'groups',
                       group_id, self.collection_key, ref['id'],
                       'inherited_to_projects'],
                      status_code=204)

        self.manager.grant(role=ref['id'], project=project_id, group=group_id,
                           os_inherit_extension_inherited=True)

    def test_domain_role_list(self):
        user_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref_list = [self.new_ref(), self.new_ref()]

        self.stub_entity('GET',
                         ['domains', domain_id, 'users', user_id,
                          self.collection_key], entity=ref_list)

        self.manager.list(domain=domain_id, user=user_id)

    def test_domain_role_list_inherited(self):
        user_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref_list = [self.new_ref(), self.new_ref()]

        self.stub_entity('GET',
                         ['OS-INHERIT',
                          'domains', domain_id, 'users', user_id,
                          self.collection_key, 'inherited_to_projects'],
                         entity=ref_list)

        returned_list = self.manager.list(domain=domain_id, user=user_id,
                                          os_inherit_extension_inherited=True)

        self.assertThat(ref_list, matchers.HasLength(len(returned_list)))
        [self.assertIsInstance(r, self.model) for r in returned_list]

    def test_project_user_role_list_inherited(self):
        user_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref_list = [self.new_ref(), self.new_ref()]

        self.stub_entity('GET',
                         ['OS-INHERIT',
                          'projects', project_id, 'users', user_id,
                          self.collection_key, 'inherited_to_projects'],
                         entity=ref_list)

        returned_list = self.manager.list(project=project_id, user=user_id,
                                          os_inherit_extension_inherited=True)

        self.assertThat(ref_list, matchers.HasLength(len(returned_list)))
        [self.assertIsInstance(r, self.model) for r in returned_list]

    def test_domain_group_role_list(self):
        group_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref_list = [self.new_ref(), self.new_ref()]

        self.stub_entity('GET',
                         ['domains', domain_id, 'groups', group_id,
                          self.collection_key], entity=ref_list)

        self.manager.list(domain=domain_id, group=group_id)

    def test_domain_group_role_list_inherited(self):
        group_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref_list = [self.new_ref(), self.new_ref()]

        self.stub_entity('GET',
                         ['OS-INHERIT',
                          'domains', domain_id, 'groups', group_id,
                          self.collection_key, 'inherited_to_projects'],
                         entity=ref_list)

        returned_list = self.manager.list(domain=domain_id, group=group_id,
                                          os_inherit_extension_inherited=True)

        self.assertThat(ref_list, matchers.HasLength(len(returned_list)))
        [self.assertIsInstance(r, self.model) for r in returned_list]

    def test_project_group_role_list_inherited(self):
        group_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref_list = [self.new_ref(), self.new_ref()]

        self.stub_entity('GET',
                         ['OS-INHERIT',
                          'projects', project_id, 'groups', group_id,
                          self.collection_key, 'inherited_to_projects'],
                         entity=ref_list)

        returned_list = self.manager.list(project=project_id, group=group_id,
                                          os_inherit_extension_inherited=True)

        self.assertThat(ref_list, matchers.HasLength(len(returned_list)))
        [self.assertIsInstance(r, self.model) for r in returned_list]

    def test_domain_role_check(self):
        user_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('HEAD',
                      ['domains', domain_id, 'users', user_id,
                       self.collection_key, ref['id']],
                      status_code=204)

        self.manager.check(role=ref['id'], domain=domain_id,
                           user=user_id)

    def test_domain_role_check_inherited(self):
        user_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('HEAD',
                      ['OS-INHERIT',
                       'domains', domain_id, 'users', user_id,
                       self.collection_key, ref['id'],
                       'inherited_to_projects'],
                      status_code=204)

        self.manager.check(role=ref['id'], domain=domain_id,
                           user=user_id, os_inherit_extension_inherited=True)

    def test_project_role_check_inherited(self):
        user_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('HEAD',
                      ['OS-INHERIT',
                       'projects', project_id, 'users', user_id,
                       self.collection_key, ref['id'],
                       'inherited_to_projects'],
                      status_code=204)

        self.manager.check(role=ref['id'], project=project_id,
                           user=user_id, os_inherit_extension_inherited=True)

    def test_domain_group_role_check(self):
        return
        group_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('HEAD',
                      ['domains', domain_id, 'groups', group_id,
                       self.collection_key, ref['id']],
                      status_code=204)

        self.manager.check(role=ref['id'], domain=domain_id, group=group_id)

    def test_domain_group_role_check_inherited(self):
        group_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('HEAD',
                      ['OS-INHERIT',
                       'domains', domain_id, 'groups', group_id,
                       self.collection_key, ref['id'],
                       'inherited_to_projects'],
                      status_code=204)

        self.manager.check(role=ref['id'], domain=domain_id,
                           group=group_id, os_inherit_extension_inherited=True)

    def test_project_group_role_check_inherited(self):
        group_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('HEAD',
                      ['OS-INHERIT',
                       'projects', project_id, 'groups', group_id,
                       self.collection_key, ref['id'],
                       'inherited_to_projects'],
                      status_code=204)

        self.manager.check(role=ref['id'], project=project_id,
                           group=group_id, os_inherit_extension_inherited=True)

    def test_domain_role_revoke(self):
        user_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('DELETE',
                      ['domains', domain_id, 'users', user_id,
                       self.collection_key, ref['id']],
                      status_code=204)

        self.manager.revoke(role=ref['id'], domain=domain_id, user=user_id)

    def test_domain_group_role_revoke(self):
        group_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('DELETE',
                      ['domains', domain_id, 'groups', group_id,
                       self.collection_key, ref['id']],
                      status_code=204)

        self.manager.revoke(role=ref['id'], domain=domain_id, group=group_id)

    def test_domain_role_revoke_inherited(self):
        user_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('DELETE',
                      ['OS-INHERIT', 'domains', domain_id, 'users', user_id,
                       self.collection_key, ref['id'],
                       'inherited_to_projects'],
                      status_code=204)

        self.manager.revoke(role=ref['id'], domain=domain_id,
                            user=user_id, os_inherit_extension_inherited=True)

    def test_project_role_revoke_inherited(self):
        user_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('DELETE',
                      ['OS-INHERIT', 'projects', project_id, 'users', user_id,
                       self.collection_key, ref['id'],
                       'inherited_to_projects'],
                      status_code=204)

        self.manager.revoke(role=ref['id'], project=project_id,
                            user=user_id, os_inherit_extension_inherited=True)

    def test_domain_group_role_revoke_inherited(self):
        group_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('DELETE',
                      ['OS-INHERIT', 'domains', domain_id, 'groups', group_id,
                       self.collection_key, ref['id'],
                       'inherited_to_projects'],
                      status_code=200)

        self.manager.revoke(role=ref['id'], domain=domain_id,
                            group=group_id,
                            os_inherit_extension_inherited=True)

    def test_project_group_role_revoke_inherited(self):
        group_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('DELETE',
                      ['OS-INHERIT', 'projects', project_id, 'groups',
                       group_id, self.collection_key, ref['id'],
                       'inherited_to_projects'],
                      status_code=204)

        self.manager.revoke(role=ref['id'], project=project_id,
                            group=group_id,
                            os_inherit_extension_inherited=True)

    def test_project_role_grant(self):
        user_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('PUT',
                      ['projects', project_id, 'users', user_id,
                       self.collection_key, ref['id']],
                      status_code=201)

        self.manager.grant(role=ref['id'], project=project_id, user=user_id)

    def test_project_group_role_grant(self):
        group_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('PUT',
                      ['projects', project_id, 'groups', group_id,
                       self.collection_key, ref['id']],
                      status_code=201)

        self.manager.grant(role=ref['id'], project=project_id, group=group_id)

    def test_project_role_list(self):
        user_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref_list = [self.new_ref(), self.new_ref()]

        self.stub_entity('GET',
                         ['projects', project_id, 'users', user_id,
                          self.collection_key], entity=ref_list)

        self.manager.list(project=project_id, user=user_id)

    def test_project_group_role_list(self):
        group_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref_list = [self.new_ref(), self.new_ref()]

        self.stub_entity('GET',
                         ['projects', project_id, 'groups', group_id,
                          self.collection_key], entity=ref_list)

        self.manager.list(project=project_id, group=group_id)

    def test_project_role_check(self):
        user_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('HEAD',
                      ['projects', project_id, 'users', user_id,
                       self.collection_key, ref['id']],
                      status_code=200)

        self.manager.check(role=ref['id'], project=project_id, user=user_id)

    def test_project_group_role_check(self):
        group_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('HEAD',
                      ['projects', project_id, 'groups', group_id,
                       self.collection_key, ref['id']],
                      status_code=200)

        self.manager.check(role=ref['id'], project=project_id, group=group_id)

    def test_project_role_revoke(self):
        user_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('DELETE',
                      ['projects', project_id, 'users', user_id,
                       self.collection_key, ref['id']],
                      status_code=204)

        self.manager.revoke(role=ref['id'], project=project_id, user=user_id)

    def test_project_group_role_revoke(self):
        group_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.stub_url('DELETE',
                      ['projects', project_id, 'groups', group_id,
                       self.collection_key, ref['id']],
                      status_code=204)

        self.manager.revoke(role=ref['id'], project=project_id, group=group_id)

    def test_domain_project_role_grant_fails(self):
        user_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.assertRaises(
            exceptions.ValidationError,
            self.manager.grant,
            role=ref['id'],
            domain=domain_id,
            project=project_id,
            user=user_id)

    def test_domain_project_role_list_fails(self):
        user_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex

        self.assertRaises(
            exceptions.ValidationError,
            self.manager.list,
            domain=domain_id,
            project=project_id,
            user=user_id)

    def test_domain_project_role_check_fails(self):
        user_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.assertRaises(
            exceptions.ValidationError,
            self.manager.check,
            role=ref['id'],
            domain=domain_id,
            project=project_id,
            user=user_id)

    def test_domain_project_role_revoke_fails(self):
        user_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        domain_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.assertRaises(
            exceptions.ValidationError,
            self.manager.revoke,
            role=ref['id'],
            domain=domain_id,
            project=project_id,
            user=user_id)

    def test_user_group_role_grant_fails(self):
        user_id = uuid.uuid4().hex
        group_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.assertRaises(
            exceptions.ValidationError,
            self.manager.grant,
            role=ref['id'],
            project=project_id,
            group=group_id,
            user=user_id)

    def test_user_group_role_list_fails(self):
        user_id = uuid.uuid4().hex
        group_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex

        self.assertRaises(
            exceptions.ValidationError,
            self.manager.list,
            project=project_id,
            group=group_id,
            user=user_id)

    def test_user_group_role_check_fails(self):
        user_id = uuid.uuid4().hex
        group_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.assertRaises(
            exceptions.ValidationError,
            self.manager.check,
            role=ref['id'],
            project=project_id,
            group=group_id,
            user=user_id)

    def test_user_group_role_revoke_fails(self):
        user_id = uuid.uuid4().hex
        group_id = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        ref = self.new_ref()

        self.assertRaises(
            exceptions.ValidationError,
            self.manager.revoke,
            role=ref['id'],
            project=project_id,
            group=group_id,
            user=user_id)


class DeprecatedImpliedRoleTests(utils.ClientTestCase):
    def setUp(self):
        super(DeprecatedImpliedRoleTests, self).setUp()
        self.key = 'role'
        self.collection_key = 'roles'
        self.model = roles.Role
        self.manager = self.client.roles

    def test_implied_create(self):
        prior_id = uuid.uuid4().hex
        prior_name = uuid.uuid4().hex
        implied_id = uuid.uuid4().hex
        implied_name = uuid.uuid4().hex

        mock_response = {
            "role_inference": {
                "implies": {
                    "id": implied_id,
                    "links": {"self": "http://host/v3/roles/%s" % implied_id},
                    "name": implied_name
                },
                "prior_role": {
                    "id": prior_id,
                    "links": {"self": "http://host/v3/roles/%s" % prior_id},
                    "name": prior_name
                }
            }
        }

        self.stub_url('PUT',
                      ['roles', prior_id, 'implies', implied_id],
                      json=mock_response,
                      status_code=201)

        with self.deprecations.expect_deprecations_here():
            manager_result = self.manager.create_implied(prior_id, implied_id)
            self.assertIsInstance(manager_result, roles.InferenceRule)
            self.assertEqual(mock_response['role_inference']['implies'],
                             manager_result.implies)
            self.assertEqual(mock_response['role_inference']['prior_role'],
                             manager_result.prior_role)


class ImpliedRoleTests(utils.ClientTestCase, utils.CrudTests):
    def setUp(self):
        super(ImpliedRoleTests, self).setUp()
        self.key = 'role_inference'
        self.collection_key = 'role_inferences'
        self.model = roles.InferenceRule
        self.manager = self.client.inference_rules

    def test_check(self):
        prior_role_id = uuid.uuid4().hex
        implied_role_id = uuid.uuid4().hex
        self.stub_url('HEAD',
                      ['roles', prior_role_id, 'implies', implied_role_id],
                      status_code=204)

        result = self.manager.check(prior_role_id, implied_role_id)
        self.assertTrue(result)

    def test_get(self):
        prior_id = uuid.uuid4().hex
        prior_name = uuid.uuid4().hex
        implied_id = uuid.uuid4().hex
        implied_name = uuid.uuid4().hex

        mock_response = {
            "role_inference": {
                "implies": {
                    "id": implied_id,
                    "links": {"self": "http://host/v3/roles/%s" % implied_id},
                    "name": implied_name
                },
                "prior_role": {
                    "id": prior_id,
                    "links": {"self": "http://host/v3/roles/%s" % prior_id},
                    "name": prior_name
                }
            }
        }

        self.stub_url('GET',
                      ['roles', prior_id, 'implies', implied_id],
                      json=mock_response,
                      status_code=200)

        manager_result = self.manager.get(prior_id, implied_id)
        self.assertIsInstance(manager_result, roles.InferenceRule)
        self.assertEqual(mock_response['role_inference']['implies'],
                         manager_result.implies)
        self.assertEqual(mock_response['role_inference']['prior_role'],
                         manager_result.prior_role)

    def test_create(self):
        prior_id = uuid.uuid4().hex
        prior_name = uuid.uuid4().hex
        implied_id = uuid.uuid4().hex
        implied_name = uuid.uuid4().hex

        mock_response = {
            "role_inference": {
                "implies": {
                    "id": implied_id,
                    "links": {"self": "http://host/v3/roles/%s" % implied_id},
                    "name": implied_name
                },
                "prior_role": {
                    "id": prior_id,
                    "links": {"self": "http://host/v3/roles/%s" % prior_id},
                    "name": prior_name
                }
            }
        }

        self.stub_url('PUT',
                      ['roles', prior_id, 'implies', implied_id],
                      json=mock_response,
                      status_code=201)

        manager_result = self.manager.create(prior_id, implied_id)

        self.assertIsInstance(manager_result, roles.InferenceRule)
        self.assertEqual(mock_response['role_inference']['implies'],
                         manager_result.implies)
        self.assertEqual(mock_response['role_inference']['prior_role'],
                         manager_result.prior_role)

    def test_delete(self):
        prior_role_id = uuid.uuid4().hex
        implied_role_id = uuid.uuid4().hex
        self.stub_url('DELETE',
                      ['roles', prior_role_id, 'implies', implied_role_id],
                      status_code=204)

        status, body = self.manager.delete(prior_role_id, implied_role_id)
        self.assertEqual(204, status.status_code)
        self.assertIsNone(body)

    def test_list_role_inferences(self):
        prior_id = uuid.uuid4().hex
        prior_name = uuid.uuid4().hex
        implied_id = uuid.uuid4().hex
        implied_name = uuid.uuid4().hex

        mock_response = {
            "role_inferences": [{
                "implies": [{
                    "id": implied_id,
                    "links": {"self": "http://host/v3/roles/%s" % implied_id},
                    "name": implied_name
                }],
                "prior_role": {
                    "id": prior_id,
                    "links": {"self": "http://host/v3/roles/%s" % prior_id},
                    "name": prior_name
                }
            }]
        }

        self.stub_url('GET',
                      ['role_inferences'],
                      json=mock_response,
                      status_code=200)
        manager_result = self.manager.list_inference_roles()
        self.assertEqual(1, len(manager_result))
        self.assertIsInstance(manager_result[0], roles.InferenceRule)
        self.assertEqual(mock_response['role_inferences'][0]['implies'],
                         manager_result[0].implies)
        self.assertEqual(mock_response['role_inferences'][0]['prior_role'],
                         manager_result[0].prior_role)

    def test_list(self):
        prior_id = uuid.uuid4().hex
        prior_name = uuid.uuid4().hex
        implied_id = uuid.uuid4().hex
        implied_name = uuid.uuid4().hex

        mock_response = {
            "role_inference": {
                "implies": [{
                    "id": implied_id,
                    "links": {"self": "http://host/v3/roles/%s" % implied_id},
                    "name": implied_name
                }],
                "prior_role": {
                    "id": prior_id,
                    "links": {"self": "http://host/v3/roles/%s" % prior_id},
                    "name": prior_name
                }
            },
            "links": {"self": "http://host/v3/roles/%s/implies" % prior_id}
        }

        self.stub_url('GET',
                      ['roles', prior_id, 'implies'],
                      json=mock_response,
                      status_code=200)

        manager_result = self.manager.list(prior_id)
        self.assertIsInstance(manager_result, roles.InferenceRule)
        self.assertEqual(1, len(manager_result.implies))
        self.assertEqual(mock_response['role_inference']['implies'],
                         manager_result.implies)
        self.assertEqual(mock_response['role_inference']['prior_role'],
                         manager_result.prior_role)

    def test_update(self):
        # Update not supported for rule inferences
        self.assertRaises(exceptions.MethodNotImplemented, self.manager.update)

    def test_find(self):
        # Find not supported for rule inferences
        self.assertRaises(exceptions.MethodNotImplemented, self.manager.find)

    def test_put(self):
        # Put not supported for rule inferences
        self.assertRaises(exceptions.MethodNotImplemented, self.manager.put)

    def test_list_params(self):
        # Put not supported for rule inferences
        self.skipTest("list params not supported by rule inferences")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import fixtures
import uuid


RESOURCE_NAME_PREFIX = 'keystoneclient-functional-'


class Base(fixtures.Fixture):

    def __init__(self, client, domain_id=None):
        super(Base, self).__init__()

        self.client = client
        self.domain_id = domain_id
        self.ref = None
        self.entity = None

    def __getattr__(self, name):
        """Return the attribute from the represented entity."""
        return getattr(self.entity, name)


class User(Base):

    def setUp(self):
        super(User, self).setUp()

        self.ref = {'name': RESOURCE_NAME_PREFIX + uuid.uuid4().hex,
                    'domain': self.domain_id}
        self.entity = self.client.users.create(**self.ref)
        self.addCleanup(self.client.users.delete, self.entity)


class Group(Base):

    def setUp(self):
        super(Group, self).setUp()

        self.ref = {'name': RESOURCE_NAME_PREFIX + uuid.uuid4().hex,
                    'domain': self.domain_id}
        self.entity = self.client.groups.create(**self.ref)
        self.addCleanup(self.client.groups.delete, self.entity)


class Domain(Base):

    def setUp(self):
        super(Domain, self).setUp()

        self.ref = {'name': RESOURCE_NAME_PREFIX + uuid.uuid4().hex,
                    'description': uuid.uuid4().hex,
                    'enabled': True}
        self.entity = self.client.domains.create(**self.ref)

        # Only disabled domains can be deleted
        self.addCleanup(self.client.domains.delete, self.entity)
        self.addCleanup(self.client.domains.update, self.entity, enabled=False)


class Project(Base):

    def __init__(self, client, domain_id=None, parent=None, tags=None):
        super(Project, self).__init__(client, domain_id)
        self.parent = parent
        self.tags = tags if tags else []

    def setUp(self):
        super(Project, self).setUp()

        self.ref = {'name': RESOURCE_NAME_PREFIX + uuid.uuid4().hex,
                    'domain': self.domain_id,
                    'enabled': True,
                    'parent': self.parent,
                    'tags': self.tags}
        self.entity = self.client.projects.create(**self.ref)
        self.addCleanup(self.client.projects.delete, self.entity)


class Role(Base):

    def __init__(self, client, name=None, domain=None):
        super(Role, self).__init__(client)
        self.name = name or RESOURCE_NAME_PREFIX + uuid.uuid4().hex
        self.domain = domain

    def setUp(self):
        super(Role, self).setUp()

        self.ref = {'name': self.name,
                    'domain': self.domain}
        self.entity = self.client.roles.create(**self.ref)
        self.addCleanup(self.client.roles.delete, self.entity)


class InferenceRule(Base):

    def __init__(self, client, prior_role, implied_role):
        super(InferenceRule, self).__init__(client)
        self.prior_role = prior_role
        self.implied_role = implied_role

    def setUp(self):
        super(InferenceRule, self).setUp()

        self.ref = {'prior_role': self.prior_role,
                    'implied_role': self.implied_role}
        self.entity = self.client.inference_rules.create(**self.ref)
        self.addCleanup(self.client.inference_rules.delete, self.prior_role,
                        self.implied_role)


class Service(Base):

    def setUp(self):
        super(Service, self).setUp()

        self.ref = {'name': RESOURCE_NAME_PREFIX + uuid.uuid4().hex,
                    'type': uuid.uuid4().hex,
                    'enabled': True,
                    'description': uuid.uuid4().hex}
        self.entity = self.client.services.create(**self.ref)
        self.addCleanup(self.client.services.delete, self.entity)


class Policy(Base):

    def setUp(self):
        super(Policy, self).setUp()

        self.ref = {'blob': uuid.uuid4().hex,
                    'type': uuid.uuid4().hex}
        self.entity = self.client.policies.create(**self.ref)
        self.addCleanup(self.client.policies.delete, self.entity)


class Region(Base):

    def __init__(self, client, parent_region=None):
        super(Region, self).__init__(client)
        self.parent_region = parent_region

    def setUp(self):
        super(Region, self).setUp()

        self.ref = {'description': uuid.uuid4().hex,
                    'parent_region': self.parent_region}
        self.entity = self.client.regions.create(**self.ref)
        self.addCleanup(self.client.regions.delete, self.entity)


class Endpoint(Base):

    def __init__(self, client, service, interface, region=None):
        super(Endpoint, self).__init__(client)
        self.service = service
        self.interface = interface
        self.region = region

    def setUp(self):
        super(Endpoint, self).setUp()

        self.ref = {'service': self.service,
                    'url': 'http://' + uuid.uuid4().hex,
                    'enabled': True,
                    'interface': self.interface,
                    'region': self.region}
        self.entity = self.client.endpoints.create(**self.ref)
        self.addCleanup(self.client.endpoints.delete, self.entity)


class EndpointGroup(Base):

    def setUp(self):
        super(EndpointGroup, self).setUp()

        self.ref = {'name': RESOURCE_NAME_PREFIX + uuid.uuid4().hex,
                    'filters': {'interface': 'public'},
                    'description': uuid.uuid4().hex}
        self.entity = self.client.endpoint_groups.create(**self.ref)
        self.addCleanup(self.client.endpoint_groups.delete, self.entity)


class Credential(Base):

    def __init__(self, client, user, type, project=None):
        super(Credential, self).__init__(client)
        self.user = user
        self.type = type
        self.project = project

        if type == 'ec2':
            self.blob = ("{\"access\":\"" + uuid.uuid4().hex +
                         "\",\"secret\":\"secretKey\"}")
        else:
            self.blob = uuid.uuid4().hex

    def setUp(self):
        super(Credential, self).setUp()

        self.ref = {'user': self.user,
                    'type': self.type,
                    'blob': self.blob,
                    'project': self.project}
        self.entity = self.client.credentials.create(**self.ref)
        self.addCleanup(self.client.credentials.delete, self.entity)


class EC2(Base):

    def __init__(self, client, user_id, project_id):
        super(EC2, self).__init__(client)
        self.user_id = user_id
        self.project_id = project_id

    def setUp(self):
        super(EC2, self).setUp()

        self.ref = {'user_id': self.user_id,
                    'project_id': self.project_id}
        self.entity = self.client.ec2.create(**self.ref)
        self.addCleanup(self.client.ec2.delete,
                        self.user_id,
                        self.entity.access)


class DomainConfig(Base):

    def __init__(self, client, domain_id):
        super(DomainConfig, self).__init__(client, domain_id=domain_id)
        self.domain_id = domain_id

    def setUp(self):
        super(DomainConfig, self).setUp()

        self.ref = {'identity': {'driver': uuid.uuid4().hex},
                    'ldap': {'url': uuid.uuid4().hex}}
        self.entity = self.client.domain_configs.create(
            self.domain_id, self.ref)
        self.addCleanup(self.client.domain_configs.delete,
                        self.domain_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from keystoneauth1.exceptions import http

from keystoneclient.tests.functional import base
from keystoneclient.tests.functional.v3 import client_fixtures as fixtures


class EndpointGroupsTestMixin(object):

    def check_endpoint_group(self, endpoint_group, endpoint_group_ref=None):
        self.assertIsNotNone(endpoint_group.id)
        self.assertIn('self', endpoint_group.links)
        self.assertIn('/endpoint_groups/' + endpoint_group.id,
                      endpoint_group.links['self'])

        if endpoint_group_ref:
            self.assertEqual(endpoint_group_ref['name'], endpoint_group.name)
            self.assertEqual(endpoint_group_ref['filters'],
                             endpoint_group.filters)

            # There is no guarantee description is present in endpoint groups
            if hasattr(endpoint_group_ref, 'description'):
                self.assertEqual(endpoint_group_ref['description'],
                                 endpoint_group.description)
        else:
            # Only check remaining mandatory attributes
            self.assertIsNotNone(endpoint_group.name)
            self.assertIsNotNone(endpoint_group.filters)


class EndpointGroupsTestCase(base.V3ClientTestCase, EndpointGroupsTestMixin):

    def test_create_endpoint_group(self):
        endpoint_group_ref = {
            'name': fixtures.RESOURCE_NAME_PREFIX + uuid.uuid4().hex,
            'filters': {'interface': 'internal'},
            'description': uuid.uuid4().hex}
        endpoint_group = self.client.endpoint_groups.create(
            **endpoint_group_ref)

        self.addCleanup(self.client.endpoint_groups.delete, endpoint_group)
        self.check_endpoint_group(endpoint_group, endpoint_group_ref)

    def test_get_endpoint_group(self):
        endpoint_group = fixtures.EndpointGroup(self.client)
        self.useFixture(endpoint_group)

        endpoint_ret = self.client.endpoint_groups.get(endpoint_group.id)
        self.check_endpoint_group(endpoint_ret, endpoint_group.ref)

        self.assertRaises(http.NotFound,
                          self.client.endpoint_groups.get,
                          uuid.uuid4().hex)

    def test_check_endpoint_group(self):
        endpoint_group = fixtures.EndpointGroup(self.client)
        self.useFixture(endpoint_group)

        self.client.endpoint_groups.check(endpoint_group.id)
        self.assertRaises(http.NotFound,
                          self.client.endpoint_groups.check,
                          uuid.uuid4().hex)

    def test_list_endpoint_groups(self):
        endpoint_group_one = fixtures.EndpointGroup(self.client)
        self.useFixture(endpoint_group_one)

        endpoint_group_two = fixtures.EndpointGroup(self.client)
        self.useFixture(endpoint_group_two)

        endpoint_groups = self.client.endpoint_groups.list()

        # All endpoints are valid
        for endpoint_group in endpoint_groups:
            self.check_endpoint_group(endpoint_group)

        self.assertIn(endpoint_group_one.entity, endpoint_groups)
        self.assertIn(endpoint_group_two.entity, endpoint_groups)

    def test_update_endpoint_group(self):
        endpoint_group = fixtures.EndpointGroup(self.client)
        self.useFixture(endpoint_group)

        new_name = fixtures.RESOURCE_NAME_PREFIX + uuid.uuid4().hex
        new_filters = {'interface': 'public'}
        new_description = uuid.uuid4().hex

        endpoint_group_ret = self.client.endpoint_groups.update(
            endpoint_group,
            name=new_name,
            filters=new_filters,
            description=new_description)

        endpoint_group.ref.update({'name': new_name, 'filters': new_filters,
                                   'description': new_description})
        self.check_endpoint_group(endpoint_group_ret, endpoint_group.ref)

    def test_delete_endpoint_group(self):
        endpoint_group = self.client.endpoint_groups.create(
            name=fixtures.RESOURCE_NAME_PREFIX + uuid.uuid4().hex,
            filters={'interface': 'admin'},
            description=uuid.uuid4().hex)

        self.client.endpoint_groups.delete(endpoint_group.id)
        self.assertRaises(http.NotFound,
                          self.client.endpoint_groups.check,
                          endpoint_group.id)
        self.assertRaises(http.NotFound,
                          self.client.endpoint_groups.get,
                          endpoint_group.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from keystoneauth1.exceptions import http
from keystoneclient.tests.functional import base
from keystoneclient.tests.functional.v3 import client_fixtures as fixtures


class DomainConfigsTestCase(base.V3ClientTestCase):

    def setUp(self):
        super(DomainConfigsTestCase, self).setUp()
        self.test_domain = fixtures.Domain(self.client)
        self.useFixture(self.test_domain)

    def check_domain_config(self, config, config_ref):
        for attr in config_ref:
            self.assertEqual(
                getattr(config, attr),
                config_ref[attr],
                'Expected different %s' % attr)

    def _new_ref(self):
        return {'identity': {'driver': uuid.uuid4().hex},
                'ldap': {'url': uuid.uuid4().hex}}

    def test_create_domain_config(self):
        config_ref = self._new_ref()
        config = self.client.domain_configs.create(
            self.test_domain.id, config_ref)
        self.addCleanup(
            self.client.domain_configs.delete, self.test_domain.id)
        self.check_domain_config(config, config_ref)

    def test_create_invalid_domain_config(self):
        invalid_groups_ref = {
            uuid.uuid4().hex: {uuid.uuid4().hex: uuid.uuid4().hex},
            uuid.uuid4().hex: {uuid.uuid4().hex: uuid.uuid4().hex}}
        self.assertRaises(http.Forbidden,
                          self.client.domain_configs.create,
                          self.test_domain.id,
                          invalid_groups_ref)

        invalid_options_ref = {
            'identity': {uuid.uuid4().hex: uuid.uuid4().hex},
            'ldap': {uuid.uuid4().hex: uuid.uuid4().hex}}
        self.assertRaises(http.Forbidden,
                          self.client.domain_configs.create,
                          self.test_domain.id,
                          invalid_options_ref)

    def test_get_domain_config(self):
        config = fixtures.DomainConfig(self.client, self.test_domain.id)
        self.useFixture(config)

        config_ret = self.client.domain_configs.get(self.test_domain.id)
        self.check_domain_config(config_ret, config.ref)

    def test_update_domain_config(self):
        config = fixtures.DomainConfig(self.client, self.test_domain.id)
        self.useFixture(config)

        update_config_ref = self._new_ref()
        config_ret = self.client.domain_configs.update(
            self.test_domain.id, update_config_ref)
        self.check_domain_config(config_ret, update_config_ref)

    def test_update_invalid_domain_config(self):
        config = fixtures.DomainConfig(self.client, self.test_domain.id)
        self.useFixture(config)

        invalid_groups_ref = {
            uuid.uuid4().hex: {uuid.uuid4().hex: uuid.uuid4().hex},
            uuid.uuid4().hex: {uuid.uuid4().hex: uuid.uuid4().hex}}
        self.assertRaises(http.Forbidden,
                          self.client.domain_configs.update,
                          self.test_domain.id,
                          invalid_groups_ref)

        invalid_options_ref = {
            'identity': {uuid.uuid4().hex: uuid.uuid4().hex},
            'ldap': {uuid.uuid4().hex: uuid.uuid4().hex}}
        self.assertRaises(http.Forbidden,
                          self.client.domain_configs.update,
                          self.test_domain.id,
                          invalid_options_ref)

    def test_domain_config_delete(self):
        config_ref = self._new_ref()
        self.client.domain_configs.create(self.test_domain.id, config_ref)

        self.client.domain_configs.delete(self.test_domain.id)
        self.assertRaises(http.NotFound,
                          self.client.domain_configs.get,
                          self.project_domain_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

from keystoneclient import exceptions
from keystoneclient.tests.unit.v3 import utils
from keystoneclient.v3 import endpoints


class EndpointTests(utils.ClientTestCase, utils.CrudTests):
    def setUp(self):
        super(EndpointTests, self).setUp()
        self.key = 'endpoint'
        self.collection_key = 'endpoints'
        self.model = endpoints.Endpoint
        self.manager = self.client.endpoints

    def new_ref(self, **kwargs):
        kwargs = super(EndpointTests, self).new_ref(**kwargs)
        kwargs.setdefault('interface', 'public')
        kwargs.setdefault('region', uuid.uuid4().hex)
        kwargs.setdefault('service_id', uuid.uuid4().hex)
        kwargs.setdefault('url', uuid.uuid4().hex)
        kwargs.setdefault('enabled', True)
        return kwargs

    def test_create_public_interface(self):
        ref = self.new_ref(interface='public')
        self.test_create(ref)

    def test_create_admin_interface(self):
        ref = self.new_ref(interface='admin')
        self.test_create(ref)

    def test_create_internal_interface(self):
        ref = self.new_ref(interface='internal')
        self.test_create(ref)

    def test_create_invalid_interface(self):
        ref = self.new_ref(interface=uuid.uuid4().hex)
        self.assertRaises(exceptions.ValidationError, self.manager.create,
                          **utils.parameterize(ref))

    def test_update_public_interface(self):
        ref = self.new_ref(interface='public')
        self.test_update(ref)

    def test_update_admin_interface(self):
        ref = self.new_ref(interface='admin')
        self.test_update(ref)

    def test_update_internal_interface(self):
        ref = self.new_ref(interface='internal')
        self.test_update(ref)

    def test_update_invalid_interface(self):
        ref = self.new_ref(interface=uuid.uuid4().hex)
        ref['endpoint'] = "fake_endpoint"
        self.assertRaises(exceptions.ValidationError, self.manager.update,
                          **utils.parameterize(ref))

    def test_list_public_interface(self):
        interface = 'public'
        expected_path = 'v3/%s?interface=%s' % (self.collection_key, interface)
        self.test_list(expected_path=expected_path, interface=interface)

    def test_list_admin_interface(self):
        interface = 'admin'
        expected_path = 'v3/%s?interface=%s' % (self.collection_key, interface)
        self.test_list(expected_path=expected_path, interface=interface)

    def test_list_internal_interface(self):
        interface = 'admin'
        expected_path = 'v3/%s?interface=%s' % (self.collection_key, interface)
        self.test_list(expected_path=expected_path, interface=interface)

    def test_list_invalid_interface(self):
        interface = uuid.uuid4().hex
        expected_path = 'v3/%s?interface=%s' % (self.collection_key, interface)
        self.assertRaises(exceptions.ValidationError, self.manager.list,
                          expected_path=expected_path, interface=interface)

    def test_list_filtered_by_region(self):
        region_id = uuid.uuid4().hex
        ref_list = [self.new_ref(region=region_id),
                    self.new_ref(region=region_id)]
        expected_path = 'v3/%s?region_id=%s' % (self.collection_key, region_id)
        expected_query = {'region_id': region_id}

        # Validate passing either region or region_id result to the API call.
        self.test_list(ref_list=ref_list, expected_path=expected_path,
                       expected_query=expected_query, region=region_id)
        self.test_list(ref_list=ref_list, expected_path=expected_path,
                       expected_query=expected_query, region_id=region_id)



# Copyright 2011 OpenStack Foundation
# Copyright 2011 Nebula, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import urllib.parse

from keystoneclient import base
from keystoneclient import exceptions
from keystoneclient.i18n import _


class Project(base.Resource):
    """Represents an Identity project.

    Attributes:
        * id: a uuid that identifies the project
        * name: project name
        * description: project description
        * enabled: boolean to indicate if project is enabled
        * parent_id: a uuid representing this project's parent in hierarchy
        * parents: a list or a structured dict containing the parents of this
                   project in the hierarchy
        * subtree: a list or a structured dict containing the subtree of this
                   project in the hierarchy

    """

    def update(self, name=None, description=None, enabled=None):
        kwargs = {
            'name': name if name is not None else self.name,
            'description': (description
                            if description is not None
                            else self.description),
            'enabled': enabled if enabled is not None else self.enabled,
        }

        try:
            retval = self.manager.update(self.id, **kwargs)
            self = retval
        except Exception:
            retval = None

        return retval

    def add_tag(self, tag):
        self.manager.add_tag(self, tag)

    def update_tags(self, tags):
        return self.manager.update_tags(self, tags)

    def delete_tag(self, tag):
        self.manager.delete_tag(self, tag)

    def delete_all_tags(self):
        return self.manager.update_tags(self, [])

    def list_tags(self):
        return self.manager.list_tags(self)

    def check_tag(self, tag):
        return self.manager.check_tag(self, tag)


class ProjectManager(base.CrudManager):
    """Manager class for manipulating Identity projects."""

    resource_class = Project
    collection_key = 'projects'
    key = 'project'

    def create(self, name, domain, description=None,
               enabled=True, parent=None, **kwargs):
        """Create a project.

        :param str name: the name of the project.
        :param domain: the domain of the project.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`
        :param str description: the description of the project.
        :param bool enabled: whether the project is enabled.
        :param parent: the parent of the project in the hierarchy.
        :type parent: str or :class:`keystoneclient.v3.projects.Project`
        :param kwargs: any other attribute provided will be passed to the
                       server.

        :returns: the created project returned from server.
        :rtype: :class:`keystoneclient.v3.projects.Project`

        """
        # NOTE(rodrigods): the API must be backwards compatible, so if an
        # application was passing a 'parent_id' before as kwargs, the call
        # should not fail. If both 'parent' and 'parent_id' are provided,
        # 'parent' will be preferred.
        if parent:
            kwargs['parent_id'] = base.getid(parent)

        return super(ProjectManager, self).create(
            domain_id=base.getid(domain),
            name=name,
            description=description,
            enabled=enabled,
            **kwargs)

    def list(self, domain=None, user=None, parent=None, **kwargs):
        """List projects.

        :param domain: the domain of the projects to be filtered on.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`
        :param user: filter in projects the specified user has role
                     assignments on.
        :type user: str or :class:`keystoneclient.v3.users.User`
        :param parent: filter in projects the specified project is a parent
                       for
        :type parent: str or :class:`keystoneclient.v3.projects.Project`
        :param kwargs: any other attribute provided will filter projects on.
                       Project tags filter keyword: ``tags``, ``tags_any``,
                       ``not_tags``, and ``not_tags_any``. tag attribute type
                       string. Pass in a comma separated string to filter
                       with multiple tags.

        :returns: a list of projects.
        :rtype: list of :class:`keystoneclient.v3.projects.Project`

        """
        base_url = '/users/%s' % base.getid(user) if user else None
        projects = super(ProjectManager, self).list(
            base_url=base_url,
            domain_id=base.getid(domain),
            parent_id=base.getid(parent),
            fallback_to_auth=True,
            **kwargs)

        base_response = None
        list_data = projects
        if self.client.include_metadata:
            base_response = projects
            list_data = projects.data
            base_response.data = list_data

        for p in list_data:
            p.tags = getattr(p, 'tags', [])

        if self.client.include_metadata:
            base_response.data = list_data

        return base_response if self.client.include_metadata else list_data

    def _check_not_parents_as_ids_and_parents_as_list(self, parents_as_ids,
                                                      parents_as_list):
        if parents_as_ids and parents_as_list:
            msg = _('Specify either parents_as_ids or parents_as_list '
                    'parameters, not both')
            raise exceptions.ValidationError(msg)

    def _check_not_subtree_as_ids_and_subtree_as_list(self, subtree_as_ids,
                                                      subtree_as_list):
        if subtree_as_ids and subtree_as_list:
            msg = _('Specify either subtree_as_ids or subtree_as_list '
                    'parameters, not both')
            raise exceptions.ValidationError(msg)

    def get(self, project, subtree_as_list=False, parents_as_list=False,
            subtree_as_ids=False, parents_as_ids=False):
        """Retrieve a project.

        :param project: the project to be retrieved from the server.
        :type project: str or :class:`keystoneclient.v3.projects.Project`
        :param bool subtree_as_list: retrieve projects below this project in
                                     the hierarchy as a flat list. It only
                                     includes the projects in which the current
                                     user has role assignments on.
        :param bool parents_as_list: retrieve projects above this project in
                                     the hierarchy as a flat list. It only
                                     includes the projects in which the current
                                     user has role assignments on.
        :param bool subtree_as_ids: retrieve the IDs from the projects below
                                    this project in the hierarchy as a
                                    structured dictionary.
        :param bool parents_as_ids: retrieve the IDs from the projects above
                                    this project in the hierarchy as a
                                    structured dictionary.
        :returns: the specified project returned from server.
        :rtype: :class:`keystoneclient.v3.projects.Project`

        :raises keystoneclient.exceptions.ValidationError: if subtree_as_list
            and subtree_as_ids or parents_as_list and parents_as_ids are
            included at the same time in the call.

        """
        self._check_not_parents_as_ids_and_parents_as_list(
            parents_as_ids, parents_as_list)
        self._check_not_subtree_as_ids_and_subtree_as_list(
            subtree_as_ids, subtree_as_list)

        # According to the API spec, the query params are key only
        query_params = []
        if subtree_as_list:
            query_params.append('subtree_as_list')
        if subtree_as_ids:
            query_params.append('subtree_as_ids')
        if parents_as_list:
            query_params.append('parents_as_list')
        if parents_as_ids:
            query_params.append('parents_as_ids')

        query = self.build_key_only_query(query_params)
        dict_args = {'project_id': base.getid(project)}
        url = self.build_url(dict_args_in_out=dict_args)
        p = self._get(url + query, self.key)
        p.tags = getattr(p, 'tags', [])
        return p

    def find(self, **kwargs):
        p = super(ProjectManager, self).find(**kwargs)
        p.tags = getattr(p, 'tags', [])
        return p

    def update(self, project, name=None, domain=None, description=None,
               enabled=None, **kwargs):
        """Update a project.

        :param project: the project to be updated on the server.
        :type project: str or :class:`keystoneclient.v3.projects.Project`
        :param str name: the new name of the project.
        :param domain: the new domain of the project.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`
        :param str description: the new description of the project.
        :param bool enabled: whether the project is enabled.
        :param kwargs: any other attribute provided will be passed to server.

        :returns: the updated project returned from server.
        :rtype: :class:`keystoneclient.v3.projects.Project`

        """
        return super(ProjectManager, self).update(
            project_id=base.getid(project),
            domain_id=base.getid(domain),
            name=name,
            description=description,
            enabled=enabled,
            **kwargs)

    def delete(self, project):
        """Delete a project.

        :param project: the project to be deleted on the server.
        :type project: str or :class:`keystoneclient.v3.projects.Project`

        :returns: Response object with 204 status.
        :rtype: :class:`requests.models.Response`

        """
        return super(ProjectManager, self).delete(
            project_id=base.getid(project))

    def add_tag(self, project, tag):
        """Add a tag to a project.

        :param project: project to add a tag to.
        :param tag: str name of tag.

        """
        url = "/projects/%s/tags/%s" % (base.getid(project),
                                        urllib.parse.quote(tag))
        return self._put(url)

    def update_tags(self, project, tags):
        """Update tag list of a project.

        Replaces current tag list with list specified in tags parameter.

        :param project: project to update.
        :param tags: list of str tag names to add to the project

        :returns: list of tags

        """
        url = "/projects/%s/tags" % base.getid(project)
        for tag in tags:
            tag = urllib.parse.quote(tag)
        resp, body = self.client.put(url, body={"tags": tags})
        return self._prepare_return_value(resp, body['tags'])

    def delete_tag(self, project, tag):
        """Remove tag from project.

        :param projectd: project to remove tag from.
        :param tag: str name of tag to remove from project

        """
        return self._delete(
            "/projects/%s/tags/%s" % (base.getid(project),
                                      urllib.parse.quote(tag)))

    def list_tags(self, project):
        """List tags associated with project.

        :param project: project to list tags for.

        :returns: list of str tag names

        """
        url = "/projects/%s/tags" % base.getid(project)
        resp, body = self.client.get(url)
        return self._prepare_return_value(resp, body['tags'])

    def check_tag(self, project, tag):
        """Check if tag is associated with project.

        :param project: project to check tags for.
        :param tag: str name of tag

        :returns: true if tag is associated, false otherwise

        """
        url = "/projects/%s/tags/%s" % (base.getid(project),
                                        urllib.parse.quote(tag))
        try:
            resp, body = self.client.head(url)
            # no errors means found the tag
            return self._prepare_return_value(resp, True)
        except exceptions.HttpError as ex:
            # return false with request_id if include_metadata=True
            return self._prepare_return_value(ex.response, False)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

from keystoneclient.tests.unit.v3 import utils
from keystoneclient.v3 import credentials


class CredentialTests(utils.ClientTestCase, utils.CrudTests):
    def setUp(self):
        super(CredentialTests, self).setUp()
        self.key = 'credential'
        self.collection_key = 'credentials'
        self.model = credentials.Credential
        self.manager = self.client.credentials

    def new_ref(self, **kwargs):
        kwargs = super(CredentialTests, self).new_ref(**kwargs)
        kwargs.setdefault('blob', uuid.uuid4().hex)
        kwargs.setdefault('project_id', uuid.uuid4().hex)
        kwargs.setdefault('type', uuid.uuid4().hex)
        kwargs.setdefault('user_id', uuid.uuid4().hex)
        return kwargs



# Copyright 2014 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient import base
from keystoneclient import exceptions
from keystoneclient.i18n import _
from keystoneclient.v3 import endpoint_groups
from keystoneclient.v3 import endpoints
from keystoneclient.v3 import projects


class EndpointFilterManager(base.Manager):
    """Manager class for manipulating project-endpoint associations.

    Project-endpoint associations can be with endpoints directly or via
    endpoint groups.

    """

    OS_EP_FILTER_EXT = '/OS-EP-FILTER'

    def _build_base_url(self, project=None, endpoint=None):
        project_id = base.getid(project)
        endpoint_id = base.getid(endpoint)

        if project_id and endpoint_id:
            api_path = '/projects/%s/endpoints/%s' % (project_id, endpoint_id)
        elif project_id:
            api_path = '/projects/%s/endpoints' % (project_id)
        elif endpoint_id:
            api_path = '/endpoints/%s/projects' % (endpoint_id)
        else:
            msg = _('Must specify a project, an endpoint, or both')
            raise exceptions.ValidationError(msg)

        return self.OS_EP_FILTER_EXT + api_path

    def _build_group_base_url(self, project=None, endpoint_group=None):
        project_id = base.getid(project)
        endpoint_group_id = base.getid(endpoint_group)

        if project_id and endpoint_group_id:
            api_path = '/endpoint_groups/%s/projects/%s' % (
                endpoint_group_id, project_id)
        elif project_id:
            api_path = '/projects/%s/endpoint_groups' % (project_id)
        elif endpoint_group_id:
            api_path = '/endpoint_groups/%s/projects' % (endpoint_group_id)
        else:
            msg = _('Must specify a project, an endpoint group, or both')
            raise exceptions.ValidationError(msg)

        return self.OS_EP_FILTER_EXT + api_path

    def add_endpoint_to_project(self, project, endpoint):
        """Create a project-endpoint association."""
        if not (project and endpoint):
            raise ValueError(_('project and endpoint are required'))

        base_url = self._build_base_url(project=project,
                                        endpoint=endpoint)
        return super(EndpointFilterManager, self)._put(url=base_url)

    def delete_endpoint_from_project(self, project, endpoint):
        """Remove a project-endpoint association."""
        if not (project and endpoint):
            raise ValueError(_('project and endpoint are required'))

        base_url = self._build_base_url(project=project,
                                        endpoint=endpoint)
        return super(EndpointFilterManager, self)._delete(url=base_url)

    def check_endpoint_in_project(self, project, endpoint):
        """Check if project-endpoint association exists."""
        if not (project and endpoint):
            raise ValueError(_('project and endpoint are required'))

        base_url = self._build_base_url(project=project,
                                        endpoint=endpoint)
        return super(EndpointFilterManager, self)._head(url=base_url)

    def list_endpoints_for_project(self, project):
        """List all endpoints for a given project."""
        if not project:
            raise ValueError(_('project is required'))

        base_url = self._build_base_url(project=project)
        return super(EndpointFilterManager, self)._list(
            base_url,
            endpoints.EndpointManager.collection_key,
            obj_class=endpoints.EndpointManager.resource_class)

    def list_projects_for_endpoint(self, endpoint):
        """List all projects for a given endpoint."""
        if not endpoint:
            raise ValueError(_('endpoint is required'))

        base_url = self._build_base_url(endpoint=endpoint)
        return super(EndpointFilterManager, self)._list(
            base_url,
            projects.ProjectManager.collection_key,
            obj_class=projects.ProjectManager.resource_class)

    def add_endpoint_group_to_project(self, endpoint_group, project):
        """Create a project-endpoint group association."""
        if not (project and endpoint_group):
            raise ValueError(_('project and endpoint_group are required'))

        base_url = self._build_group_base_url(project=project,
                                              endpoint_group=endpoint_group)
        return super(EndpointFilterManager, self)._put(url=base_url)

    def delete_endpoint_group_from_project(self, endpoint_group, project):
        """Remove a project-endpoint group association."""
        if not (project and endpoint_group):
            raise ValueError(_('project and endpoint_group are required'))

        base_url = self._build_group_base_url(project=project,
                                              endpoint_group=endpoint_group)
        return super(EndpointFilterManager, self)._delete(url=base_url)

    def check_endpoint_group_in_project(self, endpoint_group, project):
        """Check if project-endpoint group association exists."""
        if not (project and endpoint_group):
            raise ValueError(_('project and endpoint_group are required'))

        base_url = self._build_group_base_url(project=project,
                                              endpoint_group=endpoint_group)
        return super(EndpointFilterManager, self)._head(url=base_url)

    def list_endpoint_groups_for_project(self, project):
        """List all endpoint groups for a given project."""
        if not project:
            raise ValueError(_('project is required'))

        base_url = self._build_group_base_url(project=project)

        return super(EndpointFilterManager, self)._list(
            base_url,
            'endpoint_groups',
            obj_class=endpoint_groups.EndpointGroupManager.resource_class)

    def list_projects_for_endpoint_group(self, endpoint_group):
        """List all projects associated with a given endpoint group."""
        if not endpoint_group:
            raise ValueError(_('endpoint_group is required'))

        base_url = self._build_group_base_url(endpoint_group=endpoint_group)
        return super(EndpointFilterManager, self)._list(
            base_url,
            projects.ProjectManager.collection_key,
            obj_class=projects.ProjectManager.resource_class)



# Copyright 2011 OpenStack Foundation
# Copyright 2011 Nebula, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient import base
import urllib.parse


class User(base.Resource):
    """Represents a Keystone user."""

    def __repr__(self):
        """Return string representation of user resource information."""
        return "<User %s>" % self._info

    def delete(self):
        return self.manager.delete(self)

    def list_roles(self, tenant=None):
        return self.manager.list_roles(self.id, base.getid(tenant))


class UserManager(base.ManagerWithFind):
    """Manager class for manipulating Keystone users."""

    resource_class = User

    def __init__(self, client, role_manager):
        super(UserManager, self).__init__(client)
        self.role_manager = role_manager

    def get(self, user):
        return self._get("/users/%s" % base.getid(user), "user")

    def update(self, user, **kwargs):
        """Update user data.

        Supported arguments include ``name``, ``email``, and ``enabled``.
        """
        # FIXME(gabriel): "tenantId" seems to be accepted by the API but
        #                 fails to actually update the default tenant.
        params = {"user": kwargs}
        url = "/users/%s" % base.getid(user)
        return self._update(url, params, "user")

    def update_enabled(self, user, enabled):
        """Update enabled-ness."""
        params = {"user": {"enabled": enabled}}

        self._update("/users/%s/OS-KSADM/enabled" % base.getid(user), params,
                     "user")

    def update_password(self, user, password):
        """Update password."""
        params = {"user": {"password": password}}

        return self._update("/users/%s/OS-KSADM/password" % base.getid(user),
                            params, "user", log=False)

    def update_own_password(self, origpasswd, passwd):
        """Update password."""
        params = {"user": {"password": passwd,
                           "original_password": origpasswd}}

        return self._update("/OS-KSCRUD/users/%s" % self.client.user_id,
                            params,
                            response_key="access",
                            method="PATCH",
                            endpoint_filter={'interface': 'public'},
                            log=False)

    def update_tenant(self, user, tenant):
        """Update default tenant."""
        params = {"user": {"tenantId": base.getid(tenant)}}

        # FIXME(ja): seems like a bad url - default tenant is an attribute
        #            not a subresource!???
        return self._update("/users/%s/OS-KSADM/tenant" % base.getid(user),
                            params, "user")

    def create(self, name, password=None, email=None,
               tenant_id=None, enabled=True):
        """Create a user."""
        params = {"user": {"name": name,
                           "password": password,
                           "tenantId": tenant_id,
                           "email": email,
                           "enabled": enabled}}
        return self._post('/users', params, "user", log=not bool(password))

    def delete(self, user):
        """Delete a user."""
        return self._delete("/users/%s" % base.getid(user))

    def list(self, tenant_id=None, limit=None, marker=None):
        """Get a list of users (optionally limited to a tenant).

        :rtype: list of :class:`User`
        """
        params = {}
        if limit:
            params['limit'] = int(limit)
        if marker:
            params['marker'] = marker

        query = ""
        if params:
            query = "?" + urllib.parse.urlencode(params)

        if not tenant_id:
            return self._list("/users%s" % query, "users")
        else:
            return self._list("/tenants/%s/users%s" % (tenant_id, query),
                              "users")

    def list_roles(self, user, tenant=None):
        return self.role_manager.roles_for_user(base.getid(user),
                                                base.getid(tenant))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from keystoneclient import client
import os_client_config

IDENTITY_CLIENT = 'identity'
OPENSTACK_CLOUDS = ('functional_admin', 'devstack-admin', 'envvars')


def get_client(version):
    """Create a keystoneclient instance to run functional tests.

    The client is instantiated via os-client-config either based on a
    clouds.yaml config file or from the environment variables.

    First, look for a 'functional_admin' cloud, as this is a cloud that the
    user may have defined for functional testing with admin credentials. If
    that is not found, check for the 'devstack-admin' cloud. Finally, fall
    back to looking for environment variables.

    """
    for cloud in OPENSTACK_CLOUDS:
        try:
            cloud_config = os_client_config.get_config(
                cloud=cloud, identity_api_version=version)
            return cloud_config.get_legacy_client(service_key=IDENTITY_CLIENT,
                                                  constructor=client.Client)

        except os_client_config.exceptions.OpenStackConfigException:
            pass

    raise Exception("Could not find any cloud definition for clouds named"
                    " functional_admin or devstack-admin. Check your"
                    " clouds.yaml file or your envvars and try again.")


class ClientTestCase(testtools.TestCase):

    def setUp(self):
        super(ClientTestCase, self).setUp()

        if not self.auth_ref.project_scoped:
            raise Exception("Could not run functional tests, which are "
                            "run based on the scope provided for "
                            "authentication. Please provide a project "
                            "scope information.")

    @property
    def client(self):
        if not hasattr(self, '_client'):
            self._client = get_client(self.version)

        return self._client

    @property
    def auth_ref(self):
        return self.client.session.auth.get_auth_ref(self.client.session)

    @property
    def project_domain_id(self):
        return self.auth_ref.project_domain_id

    @property
    def project_id(self):
        return self.client.session.get_project_id()

    @property
    def user_id(self):
        return self.client.session.get_user_id()


class V3ClientTestCase(ClientTestCase):
    version = '3'



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from keystoneclient.tests.unit.v3 import utils
from keystoneclient.v3 import registered_limits


class RegisteredLimitTests(utils.ClientTestCase, utils.CrudTests):
    def setUp(self):
        super(RegisteredLimitTests, self).setUp()
        self.key = 'registered_limit'
        self.collection_key = 'registered_limits'
        self.model = registered_limits.RegisteredLimit
        self.manager = self.client.registered_limits

    def new_ref(self, **kwargs):
        ref = {
            'id': uuid.uuid4().hex,
            'service_id': uuid.uuid4().hex,
            'resource_name': uuid.uuid4().hex,
            'default_limit': 10,
            'description': uuid.uuid4().hex
        }
        ref.update(kwargs)
        return ref

    def test_create(self):
        # This test overrides the generic test case provided by the CrudTests
        # class because the registered limits API supports creating multiple
        # limits in a single POST request. As a result, it returns the
        # registered limits as a list of all the created limits from the
        # request. This is different from what the base test_create() method
        # assumes about keystone's API. The changes here override the base test
        # to closely model how the actual registered limit API behaves.
        ref = self.new_ref()
        manager_ref = ref.copy()
        manager_ref.pop('id')
        req_ref = [manager_ref.copy()]

        self.stub_entity('POST', entity=req_ref, status_code=201)

        returned = self.manager.create(**utils.parameterize(manager_ref))
        self.assertIsInstance(returned, self.model)

        expected_limit = req_ref.pop()
        for attr in expected_limit:
            self.assertEqual(
                getattr(returned, attr),
                expected_limit[attr],
                'Expected different %s' % attr)
        self.assertEntityRequestBodyIs([expected_limit])

    def test_list_filter_by_service(self):
        service_id = uuid.uuid4().hex
        expected_query = {'service_id': service_id}
        self.test_list(expected_query=expected_query, service=service_id)

    def test_list_filter_resource_name(self):
        resource_name = uuid.uuid4().hex
        self.test_list(resource_name=resource_name)

    def test_list_filter_region(self):
        region_id = uuid.uuid4().hex
        expected_query = {'region_id': region_id}
        self.test_list(expected_query=expected_query, region=region_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import urllib.parse
import uuid

from oslo_config import fixture as config

import testtools

from keystoneclient.auth import conf
from keystoneclient.contrib.auth.v3 import oidc
from keystoneclient import session
from keystoneclient.tests.unit.v3 import utils


ACCESS_TOKEN_ENDPOINT_RESP = {"access_token": "z5H1ITZLlJVDHQXqJun",
                              "token_type": "bearer",
                              "expires_in": 3599,
                              "scope": "profile",
                              "refresh_token": "DCERsh83IAhu9bhavrp"}

KEYSTONE_TOKEN_VALUE = uuid.uuid4().hex
UNSCOPED_TOKEN = {
    "token": {
        "issued_at": "2014-06-09T09:48:59.643406Z",
        "extras": {},
        "methods": ["oidc"],
        "expires_at": "2014-06-09T10:48:59.643375Z",
        "user": {
            "OS-FEDERATION": {
                "identity_provider": {
                    "id": "bluepages"
                },
                "protocol": {
                    "id": "oidc"
                },
                "groups": [
                    {"id": "1764fa5cf69a49a4918131de5ce4af9a"}
                ]
            },
            "id": "oidc_user%40example.com",
            "name": "oidc_user@example.com"
        }
    }
}


class AuthenticateOIDCTests(utils.TestCase):

    GROUP = 'auth'

    def setUp(self):
        super(AuthenticateOIDCTests, self).setUp()

        self.deprecations.expect_deprecations()

        self.conf_fixture = self.useFixture(config.Config())
        conf.register_conf_options(self.conf_fixture.conf, group=self.GROUP)

        self.session = session.Session()

        self.IDENTITY_PROVIDER = 'bluepages'
        self.PROTOCOL = 'oidc'
        self.USER_NAME = 'oidc_user@example.com'
        self.PASSWORD = uuid.uuid4().hex
        self.CLIENT_ID = uuid.uuid4().hex
        self.CLIENT_SECRET = uuid.uuid4().hex
        self.ACCESS_TOKEN_ENDPOINT = 'https://localhost:8020/oidc/token'
        self.FEDERATION_AUTH_URL = '%s/%s' % (
            self.TEST_URL,
            'OS-FEDERATION/identity_providers/bluepages/protocols/oidc/auth')

        self.oidcplugin = oidc.OidcPassword(
            self.TEST_URL,
            self.IDENTITY_PROVIDER,
            self.PROTOCOL,
            username=self.USER_NAME,
            password=self.PASSWORD,
            client_id=self.CLIENT_ID,
            client_secret=self.CLIENT_SECRET,
            access_token_endpoint=self.ACCESS_TOKEN_ENDPOINT)

    @testtools.skip("TypeError: __init__() got an unexpected keyword"
                    " argument 'project_name'")
    def test_conf_params(self):
        """Ensure OpenID Connect config options work."""
        section = uuid.uuid4().hex
        identity_provider = uuid.uuid4().hex
        protocol = uuid.uuid4().hex
        username = uuid.uuid4().hex
        password = uuid.uuid4().hex
        client_id = uuid.uuid4().hex
        client_secret = uuid.uuid4().hex
        access_token_endpoint = uuid.uuid4().hex

        self.conf_fixture.config(auth_section=section, group=self.GROUP)
        conf.register_conf_options(self.conf_fixture.conf, group=self.GROUP)

        self.conf_fixture.register_opts(oidc.OidcPassword.get_options(),
                                        group=section)
        self.conf_fixture.config(auth_plugin='v3oidcpassword',
                                 identity_provider=identity_provider,
                                 protocol=protocol,
                                 username=username,
                                 password=password,
                                 client_id=client_id,
                                 client_secret=client_secret,
                                 access_token_endpoint=access_token_endpoint,
                                 group=section)

        a = conf.load_from_conf_options(self.conf_fixture.conf, self.GROUP)
        self.assertEqual(identity_provider, a.identity_provider)
        self.assertEqual(protocol, a.protocol)
        self.assertEqual(username, a.username)
        self.assertEqual(password, a.password)
        self.assertEqual(client_id, a.client_id)
        self.assertEqual(client_secret, a.client_secret)
        self.assertEqual(access_token_endpoint, a.access_token_endpoint)

    def test_initial_call_to_get_access_token(self):
        """Test initial call, expect JSON access token."""
        # Mock the output that creates the access token
        self.requests_mock.post(
            self.ACCESS_TOKEN_ENDPOINT,
            json=ACCESS_TOKEN_ENDPOINT_RESP)

        # Prep all the values and send the request
        grant_type = 'password'
        scope = 'profile email'
        client_auth = (self.CLIENT_ID, self.CLIENT_SECRET)
        payload = {'grant_type': grant_type, 'username': self.USER_NAME,
                   'password': self.PASSWORD, 'scope': scope}
        res = self.oidcplugin._get_access_token(self.session,
                                                client_auth,
                                                payload,
                                                self.ACCESS_TOKEN_ENDPOINT)

        # Verify the request matches the expected structure
        self.assertEqual(self.ACCESS_TOKEN_ENDPOINT, res.request.url)
        self.assertEqual('POST', res.request.method)
        encoded_payload = urllib.parse.urlencode(payload)
        self.assertEqual(encoded_payload, res.request.body)

    def test_second_call_to_protected_url(self):
        """Test subsequent call, expect Keystone token."""
        # Mock the output that creates the keystone token
        self.requests_mock.post(
            self.FEDERATION_AUTH_URL,
            json=UNSCOPED_TOKEN,
            headers={'X-Subject-Token': KEYSTONE_TOKEN_VALUE})

        # Prep all the values and send the request
        access_token = uuid.uuid4().hex
        headers = {'Authorization': 'Bearer ' + access_token}
        res = self.oidcplugin._get_keystone_token(self.session,
                                                  headers,
                                                  self.FEDERATION_AUTH_URL)

        # Verify the request matches the expected structure
        self.assertEqual(self.FEDERATION_AUTH_URL, res.request.url)
        self.assertEqual('POST', res.request.method)
        self.assertEqual(headers['Authorization'],
                         res.request.headers['Authorization'])

    def test_end_to_end_workflow(self):
        """Test full OpenID Connect workflow."""
        # Mock the output that creates the access token
        self.requests_mock.post(
            self.ACCESS_TOKEN_ENDPOINT,
            json=ACCESS_TOKEN_ENDPOINT_RESP)

        # Mock the output that creates the keystone token
        self.requests_mock.post(
            self.FEDERATION_AUTH_URL,
            json=UNSCOPED_TOKEN,
            headers={'X-Subject-Token': KEYSTONE_TOKEN_VALUE})

        response = self.oidcplugin.get_unscoped_auth_ref(self.session)
        self.assertEqual(KEYSTONE_TOKEN_VALUE, response.auth_token)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneauth1.exceptions import http

from keystoneclient.tests.functional import base
from keystoneclient.tests.functional.v3 import client_fixtures as fixtures
from keystoneclient.tests.functional.v3 import test_endpoint_groups
from keystoneclient.tests.functional.v3 import test_projects


class EndpointFiltersTestCase(base.V3ClientTestCase,
                              test_endpoint_groups.EndpointGroupsTestMixin,
                              test_projects.ProjectsTestMixin):

    def setUp(self):
        super(EndpointFiltersTestCase, self).setUp()

        self.project = fixtures.Project(self.client)
        self.endpoint_group = fixtures.EndpointGroup(self.client)
        self.useFixture(self.project)
        self.useFixture(self.endpoint_group)

        self.client.endpoint_filter.add_endpoint_group_to_project(
            self.endpoint_group, self.project)

    def test_add_endpoint_group_to_project(self):
        project = fixtures.Project(self.client)
        endpoint_group = fixtures.EndpointGroup(self.client)
        self.useFixture(project)
        self.useFixture(endpoint_group)

        self.client.endpoint_filter.add_endpoint_group_to_project(
            endpoint_group, project)
        self.client.endpoint_filter.check_endpoint_group_in_project(
            endpoint_group, project)

    def test_delete_endpoint_group_from_project(self):
        self.client.endpoint_filter.delete_endpoint_group_from_project(
            self.endpoint_group, self.project)
        self.assertRaises(
            http.NotFound,
            self.client.endpoint_filter.check_endpoint_group_in_project,
            self.endpoint_group, self.project)

    def test_list_endpoint_groups_for_project(self):
        endpoint_group_two = fixtures.EndpointGroup(self.client)
        self.useFixture(endpoint_group_two)
        self.client.endpoint_filter.add_endpoint_group_to_project(
            endpoint_group_two, self.project)

        endpoint_groups = (
            self.client.endpoint_filter.list_endpoint_groups_for_project(
                self.project
            )
        )

        for endpoint_group in endpoint_groups:
            self.check_endpoint_group(endpoint_group)

        self.assertIn(self.endpoint_group.entity, endpoint_groups)
        self.assertIn(endpoint_group_two.entity, endpoint_groups)

    def test_list_projects_for_endpoint_group(self):
        project_two = fixtures.Project(self.client)
        self.useFixture(project_two)
        self.client.endpoint_filter.add_endpoint_group_to_project(
            self.endpoint_group, project_two)

        f = self.client.endpoint_filter.list_projects_for_endpoint_group
        projects = f(self.endpoint_group)

        for project in projects:
            self.check_project(project)

        self.assertIn(self.project.entity, projects)
        self.assertIn(project_two.entity, projects)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keystoneauth1 import plugin

from keystoneclient import base
from keystoneclient.v3.contrib.oauth1 import utils

try:
    from oauthlib import oauth1
except ImportError:
    oauth1 = None


class AccessToken(base.Resource):
    pass


class AccessTokenManager(base.CrudManager):
    """Manager class for manipulating identity OAuth access tokens."""

    resource_class = AccessToken

    def create(self, consumer_key, consumer_secret, request_key,
               request_secret, verifier):
        endpoint = utils.OAUTH_PATH + '/access_token'
        oauth_client = oauth1.Client(consumer_key,
                                     client_secret=consumer_secret,
                                     resource_owner_key=request_key,
                                     resource_owner_secret=request_secret,
                                     signature_method=oauth1.SIGNATURE_HMAC,
                                     verifier=verifier)
        url = self.client.get_endpoint(interface=plugin.AUTH_INTERFACE).rstrip(
            '/')
        url, headers, body = oauth_client.sign(url + endpoint,
                                               http_method='POST')
        resp, body = self.client.post(endpoint, headers=headers)
        token = utils.get_oauth_token_from_body(resp.content)
        return self._prepare_return_value(resp,
                                          self.resource_class(self, token))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneclient.tests.functional import base
from keystoneclient.tests.functional.v3 import client_fixtures as fixtures


role_defs = ["test_admin",
             "test_id_manager",
             "test_resource_manager",
             "test_role_manager",
             "test_assignment_manager",
             "test_domain_manager",
             "test_project_manager",
             "test_catalog_manager",
             "test_policy_manager",
             "test_observer",
             "test_domain_tech_lead",
             "test_project_observer",
             "test_member"]

inference_rules = {"test_admin": "test_policy_manager",
                   "test_id_manager": "test_project_observer",
                   "test_resource_manager": "test_project_observer",
                   "test_role_manager": "test_project_observer",
                   "test_catalog_manager": "test_project_observer",
                   "test_policy_manager": "test_project_observer",
                   "test_project_observer": "test_observer",
                   "test_member": "test_observer"}


class TestImpliedRoles(base.V3ClientTestCase):

    def setUp(self):
        super(TestImpliedRoles, self).setUp()

    def test_implied_roles(self):
        initial_rule_count = (
            len(self.client.inference_rules.list_inference_roles()))

        self.create_roles()
        self.create_rules()
        rule_count = len(self.client.inference_rules.list_inference_roles())
        self.assertEqual(initial_rule_count + len(inference_rules),
                         rule_count)

    def role_dict(self):
        roles = {role.name: role.id for role in self.client.roles.list()}
        return roles

    def create_roles(self):
        for role_def in role_defs:
            role = fixtures.Role(self.client, name=role_def)
            self.useFixture(role)

    def create_rules(self):
        roles = self.role_dict()
        for prior, implied in inference_rules.items():
            rule = fixtures.InferenceRule(self.client, roles[prior],
                                          roles[implied])
            self.useFixture(rule)
